# Copyright (C) 1998-2012 by the Free Software Foundation, Inc.
#
# This file is part of GNU Mailman.
#
# GNU Mailman is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# GNU Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# GNU Mailman.  If not, see <http://www.gnu.org/licenses/>.

import os
import sys
import pwd
import grp
import errno
import optparse

from stat import *

from mailman.configuration import config
from mailman.core.i18n import _
from mailman.version import MAILMAN_VERSION


# XXX Need to check the archives/private/*/database/* files



class State:
    FIX = False
    VERBOSE = False
    ERRORS = 0

STATE = State()

DIRPERMS            = S_ISGID | S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH
QFILEPERMS          = S_ISGID | S_IRWXU | S_IRWXG
PYFILEPERMS         = S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH
ARTICLEFILEPERMS    = S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP
MBOXPERMS           = S_IRGRP | S_IWGRP | S_IRUSR | S_IWUSR
PRIVATEPERMS        = QFILEPERMS



def statmode(path):
    return os.stat(path).st_mode


def statgidmode(path):
    stat = os.stat(path)
    return stat.st_mode, stat.st_gid


seen = {}

# libc's getgrgid re-opens /etc/group each time :(
_gidcache = {}

def getgrgid(gid):
    data = _gidcache.get(gid)
    if data is None:
        data = grp.getgrgid(gid)
        _gidcache[gid] = data
    return data



def checkwalk(arg, dirname, names):
    # Short-circuit duplicates
    if seen.has_key(dirname):
        return
    seen[dirname] = True
    for name in names:
        path = os.path.join(dirname, name)
        if arg.VERBOSE:
            print _('    checking gid and mode for $path')
        try:
            mode, gid = statgidmode(path)
        except OSError, e:
            if e.errno != errno.ENOENT: raise
            continue
        if gid != MAILMAN_GID:
            try:
                groupname = getgrgid(gid)[0]
            except KeyError:
                groupname = '<anon gid %d>' % gid
            arg.ERRORS += 1
            print _(
                '$path bad group (has: $groupname, expected $MAILMAN_GROUP)'),
            if STATE.FIX:
                print _('(fixing)')
                os.chown(path, -1, MAILMAN_GID)
            else:
                print
        # Most directories must be at least rwxrwsr-x.
        # The private archive directory  and database directory must be at
        # least rwxrws---.  Their 'other' permissions are checked in
        # checkarchives() and checkarchivedbs() below.  Their 'user' and
        # 'group' permissions are checked here.
        # The directories under qfiles should be rwxrws---.  Their 'user' and
        # 'group' permissions are checked here.  Their 'other' permissions
        # aren't checked.
        private = config.PRIVATE_ARCHIVE_FILE_DIR
        if path == private or (
            os.path.commonprefix((path, private)) == private
            and os.path.split(path)[1] == 'database'):
            # then...
            targetperms = PRIVATEPERMS
        elif (os.path.commonprefix((path, config.QUEUE_DIR))
              == config.QUEUE_DIR):
            targetperms = QFILEPERMS
        else:
            targetperms = DIRPERMS
        octperms = oct(targetperms)
        if S_ISDIR(mode) and (mode & targetperms) != targetperms:
            arg.ERRORS += 1
            print _('directory permissions must be $octperms: $path'),
            if STATE.FIX:
                print _('(fixing)')
                os.chmod(path, mode | targetperms)
            else:
                print
        elif os.path.splitext(path)[1] in ('.py', '.pyc', '.pyo'):
            octperms = oct(PYFILEPERMS)
            if mode & PYFILEPERMS != PYFILEPERMS:
                print _('source perms must be $octperms: $path'),
                arg.ERRORS += 1
                if STATE.FIX:
                    print _('(fixing)')
                    os.chmod(path, mode | PYFILEPERMS)
                else:
                    print
        elif path.endswith('-article'):
            # Article files must be group writeable
            octperms = oct(ARTICLEFILEPERMS)
            if mode & ARTICLEFILEPERMS != ARTICLEFILEPERMS:
                print _('article db files must be $octperms: $path'),
                arg.ERRORS += 1
                if STATE.FIX:
                    print _('(fixing)')
                    os.chmod(path, mode | ARTICLEFILEPERMS)
                else:
                    print



def checkall():
    # first check PREFIX
    if STATE.VERBOSE:
        prefix = config.PREFIX
        print _('checking mode for $prefix')
    dirs = {}
    for d in (config.PREFIX, config.EXEC_PREFIX, config.VAR_PREFIX,
              config.LOG_DIR):
        dirs[d] = True
    for d in dirs.keys():
        try:
            mode = statmode(d)
        except OSError, e:
            if e.errno != errno.ENOENT: raise
            print _('WARNING: directory does not exist: $d')
            continue
        if (mode & DIRPERMS) != DIRPERMS:
            STATE.ERRORS += 1
            print _('directory must be at least 02775: $d'),
            if STATE.FIX:
                print _('(fixing)')
                os.chmod(d, mode | DIRPERMS)
            else:
                print
        # check all subdirs
        os.path.walk(d, checkwalk, STATE)



def checkarchives():
    private = config.PRIVATE_ARCHIVE_FILE_DIR
    if STATE.VERBOSE:
        print _('checking perms on $private')
    # private archives must not be other readable
    mode = statmode(private)
    if mode & S_IROTH:
        STATE.ERRORS += 1
        print _('$private must not be other-readable'),
        if STATE.FIX:
            print _('(fixing)')
            os.chmod(private, mode & ~S_IROTH)
        else:
            print
    # In addition, on a multiuser system you may want to hide the private
    # archives so other users can't read them.
    if mode & S_IXOTH:
        print _("""\
Warning: Private archive directory is other-executable (o+x).
         This could allow other users on your system to read private archives.
         If you're on a shared multiuser system, you should consult the
         installation manual on how to fix this.""")



def checkmboxfile(mboxdir):
    absdir = os.path.join(config.PRIVATE_ARCHIVE_FILE_DIR, mboxdir)
    for f in os.listdir(absdir):
        if not f.endswith('.mbox'):
            continue
        mboxfile = os.path.join(absdir, f)
        mode = statmode(mboxfile)
        if (mode & MBOXPERMS) != MBOXPERMS:
            STATE.ERRORS = STATE.ERRORS + 1
            print _('mbox file must be at least 0660:'), mboxfile
            if STATE.FIX:
                print _('(fixing)')
                os.chmod(mboxfile, mode | MBOXPERMS)
            else:
                print



def checkarchivedbs():
    # The archives/private/listname/database file must not be other readable
    # or executable otherwise those files will be accessible when the archives
    # are public.  That may not be a horrible breach, but let's close this off
    # anyway.
    for dir in os.listdir(config.PRIVATE_ARCHIVE_FILE_DIR):
        if dir.endswith('.mbox'):
            checkmboxfile(dir)
        dbdir = os.path.join(config.PRIVATE_ARCHIVE_FILE_DIR, dir, 'database')
        try:
            mode = statmode(dbdir)
        except OSError, e:
            if e.errno not in (errno.ENOENT, errno.ENOTDIR): raise
            continue
        if mode & S_IRWXO:
            STATE.ERRORS += 1
            print _('$dbdir "other" perms must be 000'),
            if STATE.FIX:
                print _('(fixing)')
                os.chmod(dbdir, mode & ~S_IRWXO)
            else:
                print



def checkcgi():
    cgidir = os.path.join(config.EXEC_PREFIX, 'cgi-bin')
    if STATE.VERBOSE:
        print _('checking cgi-bin permissions')
    exes = os.listdir(cgidir)
    for f in exes:
        path = os.path.join(cgidir, f)
        if STATE.VERBOSE:
            print _('    checking set-gid for $path')
        mode = statmode(path)
        if mode & S_IXGRP and not mode & S_ISGID:
            STATE.ERRORS += 1
            print _('$path must be set-gid'),
            if STATE.FIX:
                print _('(fixing)')
                os.chmod(path, mode | S_ISGID)
            else:
                print



def checkmail():
    wrapper = os.path.join(config.WRAPPER_DIR, 'mailman')
    if STATE.VERBOSE:
        print _('checking set-gid for $wrapper')
    mode = statmode(wrapper)
    if not mode & S_ISGID:
        STATE.ERRORS += 1
        print _('$wrapper must be set-gid'),
        if STATE.FIX:
            print _('(fixing)')
            os.chmod(wrapper, mode | S_ISGID)



def checkadminpw():
    for pwfile in (os.path.join(config.DATA_DIR, 'adm.pw'),
                   os.path.join(config.DATA_DIR, 'creator.pw')):
        targetmode = S_IFREG | S_IRUSR | S_IWUSR | S_IRGRP
        if STATE.VERBOSE:
            print _('checking permissions on $pwfile')
        try:
            mode = statmode(pwfile)
        except OSError, e:
            if e.errno != errno.ENOENT:
                raise
            return
        if mode != targetmode:
            STATE.ERRORS += 1
            octmode = oct(mode)
            print _('$pwfile permissions must be exactly 0640 (got $octmode)'),
            if STATE.FIX:
                print _('(fixing)')
                os.chmod(pwfile, targetmode)
            else:
                print


def checkmta():
    if config.MTA:
        modname = 'mailman.MTA.' + config.MTA
        __import__(modname)
        try:
            sys.modules[modname].checkperms(STATE)
        except AttributeError:
            pass



def checkdata():
    targetmode = S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP
    checkfiles = ('config.pck', 'config.pck.last',
                  'config.db', 'config.db.last',
                  'next-digest', 'next-digest-topics',
                  'digest.mbox', 'pending.pck',
                  'request.db', 'request.db.tmp')
    if STATE.VERBOSE:
        print _('checking permissions on list data')
    for dir in os.listdir(config.LIST_DATA_DIR):
        for file in checkfiles:
            path = os.path.join(config.LIST_DATA_DIR, dir, file)
            if STATE.VERBOSE:
                print _('    checking permissions on: $path')
            try:
                mode = statmode(path)
            except OSError, e:
                if e.errno != errno.ENOENT:
                    raise
                continue
            if (mode & targetmode) != targetmode:
                STATE.ERRORS += 1
                print _('file permissions must be at least 660: $path'),
                if STATE.FIX:
                    print _('(fixing)')
                    os.chmod(path, mode | targetmode)
                else:
                    print



def parseargs():
    parser = optparse.OptionParser(version=MAILMAN_VERSION,
                                   usage=_("""\
%prog [options]

Check the permissions of all Mailman files.  With no options, just report the
permission and ownership problems found."""))
    parser.add_option('-f', '--fix',
                      default=False, action='store_true', help=_("""\
Fix all permission and ownership problems found.  With this option, you must
run check_perms as root."""))
    parser.add_option('-v', '--verbose',
                      default=False, action='store_true',
                      help=_('Produce more verbose output'))
    parser.add_option('-C', '--config',
                      help=_('Alternative configuration file to use'))
    opts, args = parser.parse_args()
    if args:
        parser.print_help()
        print >> sys.stderr, _('Unexpected arguments')
        sys.exit(1)
    return parser, opts, args



def main():
    global MAILMAN_USER, MAILMAN_GROUP, MAILMAN_UID, MAILMAN_GID

    parser, opts, args = parseargs()
    STATE.FIX = opts.fix
    STATE.VERBOSE = opts.verbose

    config.load(opts.config)

    MAILMAN_USER  = config.MAILMAN_USER
    MAILMAN_GROUP = config.MAILMAN_GROUP
    # Let KeyErrors percolate
    MAILMAN_GID = grp.getgrnam(MAILMAN_GROUP).gr_gid
    MAILMAN_UID = pwd.getpwnam(MAILMAN_USER).pw_uid

    checkall()
    checkarchives()
    checkarchivedbs()
    checkcgi()
    checkmail()
    checkdata()
    checkadminpw()
    checkmta()

    if not STATE.ERRORS:
        print _('No problems found')
    else:
        print _('Problems found:'), STATE.ERRORS
        print _('Re-run as $MAILMAN_USER (or root) with -f flag to fix')


if __name__ == '__main__':
    main()



# Copyright (C) 2001-2012 by the Free Software Foundation, Inc.
#
# This file is part of GNU Mailman.
#
# GNU Mailman is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# GNU Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# GNU Mailman.  If not, see <http://www.gnu.org/licenses/>.

import time
import logging
import optparse

from mailman import MailList
from mailman import MemberAdaptor
from mailman import Pending
from mailman import loginit
from mailman.Bouncer import _BounceInfo
from mailman.configuration import config
from mailman.core.i18n import _
from mailman.interfaces.member import NotAMemberError
from mailman.version import MAILMAN_VERSION


# Work around known problems with some RedHat cron daemons
import signal
signal.signal(signal.SIGCHLD, signal.SIG_DFL)

ALL = (MemberAdaptor.BYBOUNCE,
       MemberAdaptor.BYADMIN,
       MemberAdaptor.BYUSER,
       MemberAdaptor.UNKNOWN,
       )



def who_callback(option, opt, value, parser):
    dest = getattr(parser.values, option.dest)
    if opt in ('-o', '--byadmin'):
        dest.add(MemberAdaptor.BYADMIN)
    elif opt in ('-m', '--byuser'):
        dest.add(MemberAdaptor.BYUSER)
    elif opt in ('-u', '--unknown'):
        dest.add(MemberAdaptor.UNKNOWN)
    elif opt in ('-b', '--notbybounce'):
        dest.discard(MemberAdaptor.BYBOUNCE)
    elif opt in ('-a', '--all'):
        dest.update(ALL)


def parseargs():
    parser = optparse.OptionParser(version=MAILMAN_VERSION,
                                   usage=_("""\
%prog [options]

Process disabled members, recommended once per day.

This script iterates through every mailing list looking for members whose
delivery is disabled.  If they have been disabled due to bounces, they will
receive another notification, or they may be removed if they've received the
maximum number of notifications.

Use the --byadmin, --byuser, and --unknown flags to also send notifications to
members whose accounts have been disabled for those reasons.  Use --all to
send the notification to all disabled members."""))
    # This is the set of working flags for who to send notifications to.  By
    # default, we notify anybody who has been disable due to bounces.
    parser.set_defaults(who=set([MemberAdaptor.BYBOUNCE]))
    parser.add_option('-o', '--byadmin',
                      callback=who_callback, action='callback', dest='who',
                      help=_("""\
Also send notifications to any member disabled by the list
owner/administrator."""))
    parser.add_option('-m', '--byuser',
                      callback=who_callback, action='callback', dest='who',
                      help=_("""\
Also send notifications to any member who has disabled themself."""))
    parser.add_option('-u', '--unknown',
                      callback=who_callback, action='callback', dest='who',
                      help=_("""\
Also send notifications to any member disabled for unknown reasons
(usually a legacy disabled address)."""))
    parser.add_option('-b', '--notbybounce',
                      callback=who_callback, action='callback', dest='who',
                      help=_("""\
Don't send notifications to members disabled because of bounces (the
default is to notify bounce disabled members)."""))
    parser.add_option('-a', '--all',
                      callback=who_callback, action='callback', dest='who',
                      help=_('Send notifications to all disabled members'))
    parser.add_option('-f', '--force',
                      default=False, action='store_true',
                      help=_("""\
Send notifications to disabled members even if they're not due a new
notification yet."""))
    parser.add_option('-l', '--listname',
                      dest='listnames', action='append', default=[],
                      type='string', help=_("""\
Process only the given list, otherwise do all lists."""))
    parser.add_option('-C', '--config',
                      help=_('Alternative configuration file to use'))
    opts, args = parser.parse_args()
    return opts, args, parser



def main():
    opts, args, parser = parseargs()
    config.load(opts.config)

    loginit.initialize(propagate=True)
    elog = logging.getLogger('mailman.error')
    blog = logging.getLogger('mailman.bounce')

    listnames = set(opts.listnames or config.list_manager.names)
    who = tuple(opts.who)

    msg = _('[disabled by periodic sweep and cull, no message available]')
    today = time.mktime(time.localtime()[:3] + (0,) * 6)
    for listname in listnames:
        # List of members to notify
        notify = []
        mlist = MailList.MailList(listname)
        try:
            interval = mlist.bounce_you_are_disabled_warnings_interval
            # Find all the members who are currently bouncing and see if
            # they've reached the disable threshold but haven't yet been
            # disabled.  This is a sweep through the membership catching
            # situations where they've bounced a bunch, then the list admin
            # lowered the threshold, but we haven't (yet) seen more bounces
            # from the member.  Note: we won't worry about stale information
            # or anything else since the normal bounce processing code will
            # handle that.
            disables = []
            for member in mlist.getBouncingMembers():
                if mlist.getDeliveryStatus(member) <> MemberAdaptor.ENABLED:
                    continue
                info = mlist.getBounceInfo(member)
                if info.score >= mlist.bounce_score_threshold:
                    disables.append((member, info))
            if disables:
                for member, info in disables:
                    mlist.disableBouncingMember(member, info, msg)
            # Go through all the members who have delivery disabled, and find
            # those that are due to have another notification.  If they are
            # disabled for another reason than bouncing, and we're processing
            # them (because of the command line switch) then they won't have a
            # bounce info record.  We can piggyback on that for all disable
            # purposes.
            members = mlist.getDeliveryStatusMembers(who)
            for member in members:
                info = mlist.getBounceInfo(member)
                if not info:
                    # See if they are bounce disabled, or disabled for some
                    # other reason.
                    status = mlist.getDeliveryStatus(member)
                    if status == MemberAdaptor.BYBOUNCE:
                        elog.error(
                            '%s disabled BYBOUNCE lacks bounce info, list: %s',
                            member, mlist.internal_name())
                        continue
                    info = _BounceInfo(
                        member, 0, today,
                        mlist.bounce_you_are_disabled_warnings,
                        mlist.pend_new(Pending.RE_ENABLE,
                                       mlist.internal_name(),
                                       member))
                    mlist.setBounceInfo(member, info)
                lastnotice = time.mktime(info.lastnotice + (0,) * 6)
                if opts.force or today >= lastnotice + interval:
                    notify.append(member)
            # Now, send notifications to anyone who is due
            for member in notify:
                blog.info('Notifying disabled member %s for list: %s',
                          member, mlist.internal_name())
                try:
                    mlist.sendNextNotification(member)
                except NotAMemberError:
                    # There must have been some problem with the data we have
                    # on this member.  Most likely it's that they don't have a
                    # password assigned.  Log this and delete the member.
                    blog.info(
                        'Cannot send disable notice to non-member: %s',
                        member)
                    mlist.ApprovedDeleteMember(member, 'cron/disabled')
            mlist.Save()
        finally:
            mlist.Unlock()



if __name__ == '__main__':
    main()



# Copyright (C) 2007-2012 by the Free Software Foundation, Inc.
#
# This file is part of GNU Mailman.
#
# GNU Mailman is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# GNU Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# GNU Mailman.  If not, see <http://www.gnu.org/licenses/>.

"""Application level list creation."""

from __future__ import absolute_import, unicode_literals

__metaclass__ = type
__all__ = [
    'create_list',
    'remove_list',
    ]


import os
import shutil
import logging

from zope.component import getUtility

from mailman.config import config
from mailman.interfaces.address import IEmailValidator
from mailman.interfaces.domain import (
    BadDomainSpecificationError, IDomainManager)
from mailman.interfaces.listmanager import IListManager
from mailman.interfaces.member import MemberRole
from mailman.interfaces.styles import IStyleManager
from mailman.interfaces.usermanager import IUserManager
from mailman.utilities.modules import call_name


log = logging.getLogger('mailman.error')



def create_list(fqdn_listname, owners=None):
    """Create the named list and apply styles.

    The mailing may not exist yet, but the domain specified in `fqdn_listname`
    must exist.

    :param fqdn_listname: The fully qualified name for the new mailing list.
    :type fqdn_listname: string
    :param owners: The mailing list owners.
    :type owners: list of string email addresses
    :return: The new mailing list.
    :rtype: `IMailingList`
    :raises BadDomainSpecificationError: when the hostname part of
        `fqdn_listname` does not exist.
    :raises ListAlreadyExistsError: when the mailing list already exists.
    :raises InvalidEmailAddressError: when the fqdn email address is invalid.
    """
    if owners is None:
        owners = []
    # This raises I
    getUtility(IEmailValidator).validate(fqdn_listname)
    listname, domain = fqdn_listname.split('@', 1)
    if domain not in getUtility(IDomainManager):
        raise BadDomainSpecificationError(domain)
    mlist = getUtility(IListManager).create(fqdn_listname)
    for style in getUtility(IStyleManager).lookup(mlist):
        style.apply(mlist)
    # Coordinate with the MTA, as defined in the configuration file.
    call_name(config.mta.incoming).create(mlist)
    # Create any owners that don't yet exist, and subscribe all addresses as
    # owners of the mailing list.
    user_manager = getUtility(IUserManager)
    for owner_address in owners:
        address = user_manager.get_address(owner_address)
        if address is None:
            user = user_manager.create_user(owner_address)
            address = list(user.addresses)[0]
        mlist.subscribe(address, MemberRole.owner)
    return mlist



def remove_list(fqdn_listname, mailing_list=None):
    """Remove the list and all associated artifacts and subscriptions."""
    removeables = []
    # mailing_list will be None when only residual archives are being removed.
    if mailing_list is not None:
        # Remove all subscriptions, regardless of role.
        for member in mailing_list.subscribers.members:
            member.unsubscribe()
        # Delete the mailing list from the database.
        getUtility(IListManager).delete(mailing_list)
        # Do the MTA-specific list deletion tasks
        call_name(config.mta.incoming).create(mailing_list)
        # Remove the list directory.
        removeables.append(os.path.join(config.LIST_DATA_DIR, fqdn_listname))
    # Remove any stale locks associated with the list.
    for filename in os.listdir(config.LOCK_DIR):
        fn_listname = filename.split('.')[0]
        if fn_listname == fqdn_listname:
            removeables.append(os.path.join(config.LOCK_DIR, filename))
    # Now that we know what files and directories to delete, delete them.
    for target in removeables:
        if not os.path.exists(target):
            pass
        elif os.path.islink(target):
            os.unlink(target)
        elif os.path.isdir(target):
            shutil.rmtree(target)
        elif os.path.isfile(target):
            os.unlink(target)
        else:
            log.error('Could not delete list artifact: %s', target)



# Copyright (C) 2012 by the Free Software Foundation, Inc.
#
# This file is part of GNU Mailman.
#
# GNU Mailman is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# GNU Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# GNU Mailman.  If not, see <http://www.gnu.org/licenses/>.

"""Load the base schema."""

from __future__ import absolute_import, print_function, unicode_literals

__metaclass__ = type
__all__ = [
    'upgrade',
    'post_reset',
    'pre_reset',
    ]


_migration_path = None
VERSION = '00000000000000'



def upgrade(database, store, version, module_path):
    filename = '{0}.sql'.format(database.TAG)
    database.load_schema(store, version, filename, module_path)


def pre_reset(store):
    global _migration_path
    # Save the entry in the Version table for the test suite reset.  This will
    # be restored below.
    from mailman.model.version import Version
    result = store.find(Version, component=VERSION).one()
    # Yes, we abuse this field.
    _migration_path = result.version


def post_reset(store):
    from mailman.model.version import Version
    # We need to preserve the Version table entry for this migration, since
    # its existence defines the fact that the tables have been loaded.
    store.add(Version(component='schema', version=VERSION))
    store.add(Version(component=VERSION, version=_migration_path))



# Copyright (C) 2011-2012 by the Free Software Foundation, Inc.
#
# This file is part of GNU Mailman.
#
# GNU Mailman is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# GNU Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# GNU Mailman.  If not, see <http://www.gnu.org/licenses/>.

"""Test users."""

from __future__ import absolute_import, unicode_literals

__metaclass__ = type
__all__ = [
    'TestUser',
    ]


import unittest

from zope.component import getUtility

from mailman.app.lifecycle import create_list
from mailman.interfaces.usermanager import IUserManager
from mailman.testing.layers import ConfigLayer
from mailman.utilities.datetime import now



class TestUser(unittest.TestCase):
    """Test users."""

    layer = ConfigLayer

    def setUp(self):
        self._mlist = create_list('test@example.com')
        self._anne = getUtility(IUserManager).create_user(
            'anne@example.com', 'Anne Person')
        preferred = list(self._anne.addresses)[0]
        preferred.verified_on = now()
        self._anne.preferred_address = preferred

    def test_preferred_address_memberships(self):
        self._mlist.subscribe(self._anne)
        memberships = list(self._anne.memberships.members)
        self.assertEqual(len(memberships), 1)
        self.assertEqual(memberships[0].address.email, 'anne@example.com')
        self.assertEqual(memberships[0].user, self._anne)
        addresses = list(self._anne.memberships.addresses)
        self.assertEqual(len(addresses), 1)
        self.assertEqual(addresses[0].email, 'anne@example.com')

    def test_preferred_and_address_memberships(self):
        self._mlist.subscribe(self._anne)
        aperson = self._anne.register('aperson@example.com')
        self._mlist.subscribe(aperson)
        memberships = list(self._anne.memberships.members)
        self.assertEqual(len(memberships), 2)
        self.assertEqual(set(member.address.email for member in memberships),
                         set(['anne@example.com', 'aperson@example.com']))
        self.assertEqual(memberships[0].user, memberships[1].user)
        self.assertEqual(memberships[0].user, self._anne)
        emails = set(address.email
                     for address in self._anne.memberships.addresses)
        self.assertEqual(len(emails), 2)
        self.assertEqual(emails,
                         set(['anne@example.com', 'aperson@example.com']))



# Copyright (C) 2009-2012 by the Free Software Foundation, Inc.
#
# This file is part of GNU Mailman.
#
# GNU Mailman is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# GNU Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# GNU Mailman.  If not, see <http://www.gnu.org/licenses/>.

"""Module stuff."""

from __future__ import absolute_import, unicode_literals

__metaclass__ = type
__all__ = [
    'Reopen',
    'Restart',
    'Start',
    'Stop',
    ]


import os
import sys
import errno
import signal
import logging

from zope.interface import implements

from mailman.bin.master import WatcherState, master_state
from mailman.config import config
from mailman.core.i18n import _
from mailman.interfaces.command import ICLISubCommand


qlog = logging.getLogger('mailman.runner')



class Start:
    """Start the Mailman daemons."""

    implements(ICLISubCommand)

    name = 'start'

    def add(self, parser, command_parser):
        """See `ICLISubCommand`."""
        self.parser = parser
        command_parser.add_argument(
            '-f', '--force',
            default=False, action='store_true',
            help=_("""\
            If the master watcher finds an existing master lock, it will
            normally exit with an error message.  With this option,the master
            will perform an extra level of checking.  If a process matching
            the host/pid described in the lock file is running, the master
            will still exit, requiring you to manually clean up the lock.  But
            if no matching process is found, the master will remove the
            apparently stale lock and make another attempt to claim the master
            lock."""))
        command_parser.add_argument(
            '-u', '--run-as-user',
            default=True, action='store_false',
            help=_("""\
            Normally, this script will refuse to run if the user id and group
            id are not set to the 'mailman' user and group (as defined when
            you configured Mailman).  If run as root, this script will change
            to this user and group before the check is made.

            This can be inconvenient for testing and debugging purposes, so
            the -u flag means that the step that sets and checks the uid/gid
            is skipped, and the program is run as the current user and group.
            This flag is not recommended for normal production environments.

            Note though, that if you run with -u and are not in the mailman
            group, you may have permission problems, such as begin unable to
            delete a list's archives through the web.  Tough luck!"""))
        command_parser.add_argument(
            '-q', '--quiet',
            default=False, action='store_true',
            help=_("""\
            Don't print status messages.  Error messages are still printed to
            standard error."""))

    def process(self, args):
        """See `ICLISubCommand`."""
        # Although there's a potential race condition here, it's a better user
        # experience for the parent process to refuse to start twice, rather
        # than having it try to start the master, which will error exit.
        status, lock = master_state()
        if status is WatcherState.conflict:
            self.parser.error(_('GNU Mailman is already running'))
        elif status in (WatcherState.stale_lock, WatcherState.host_mismatch):
            if args.force is None:
                self.parser.error(
                    _('A previous run of GNU Mailman did not exit '
                      'cleanly.  Try using --force.'))
        def log(message):
            if not args.quiet:
                print message
        # Daemon process startup according to Stevens, Advanced Programming in
        # the UNIX Environment, Chapter 13.
        pid = os.fork()
        if pid:
            # parent
            log(_("Starting Mailman's master runner"))
            return
        # child: Create a new session and become the session leader, but since
        # we won't be opening any terminal devices, don't do the
        # ultra-paranoid suggestion of doing a second fork after the setsid()
        # call.
        os.setsid()
        # Instead of cd'ing to root, cd to the Mailman runtime directory.
        # However, before we do that, set an environment variable used by the
        # subprocesses to calculate their path to the $VAR_DIR.
        os.environ['MAILMAN_VAR_DIR'] = config.VAR_DIR
        os.chdir(config.VAR_DIR)
        # Exec the master watcher.
        execl_args = [
            sys.executable, sys.executable,
            os.path.join(config.BIN_DIR, 'master'),
            ]
        if args.force:
            execl_args.append('--force')
        if args.config:
            execl_args.extend(['-C', args.config])
        qlog.debug('starting: %s', execl_args)
        os.execl(*execl_args)
        # We should never get here.
        raise RuntimeError('os.execl() failed')



def kill_watcher(sig):
    try:
        with open(config.PID_FILE) as fp:
            pid = int(fp.read().strip())
    except (IOError, ValueError) as error:
        # For i18n convenience
        print >> sys.stderr, _('PID unreadable in: $config.PID_FILE')
        print >> sys.stderr, error
        print >> sys.stderr, _('Is the master even running?')
        return
    try:
        os.kill(pid, sig)
    except OSError as error:
        if error.errno != errno.ESRCH:
            raise
        print >> sys.stderr, _('No child with pid: $pid')
        print >> sys.stderr, error
        print >> sys.stderr, _('Stale pid file removed.')
        os.unlink(config.PID_FILE)



class SignalCommand:
    """Common base class for simple, signal sending commands."""

    implements(ICLISubCommand)

    name = None
    message = None
    signal = None

    def add(self, parser, command_parser):
        """See `ICLISubCommand`."""
        command_parser.add_argument(
            '-q', '--quiet',
            default=False, action='store_true',
            help=_("""\
            Don't print status messages.  Error messages are still printed to
            standard error."""))

    def process(self, args):
        """See `ICLISubCommand`."""
        if not args.quiet:
            print _(self.message)
        kill_watcher(self.signal)


class Stop(SignalCommand):
    """Stop the Mailman daemons."""

    name = 'stop'
    message = _("Shutting down Mailman's master runner")
    signal = signal.SIGTERM


class Reopen(SignalCommand):
    """Reopen the Mailman daemons."""

    name = 'reopen'
    message = _('Reopening the Mailman runners')
    signal = signal.SIGHUP


class Restart(SignalCommand):
    """Stop the Mailman daemons."""

    implements(ICLISubCommand)

    name = 'restart'
    message = _('Restarting the Mailman runners')
    signal = signal.SIGUSR1



# Copyright (C) 1998-2012 by the Free Software Foundation, Inc.
#
# This file is part of GNU Mailman.
#
# GNU Mailman is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# GNU Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# GNU Mailman.  If not, see <http://www.gnu.org/licenses/>.

"""Send an acknowledgment of the successful post to the sender.

This only happens if the sender has set their AcknowledgePosts attribute.
"""

from __future__ import absolute_import, unicode_literals

__metaclass__ = type
__all__ = [
    'Acknowledge',
    ]


from zope.component import getUtility
from zope.interface import implements

from mailman.core.i18n import _
from mailman.email.message import UserNotification
from mailman.interfaces.handler import IHandler
from mailman.interfaces.languages import ILanguageManager
from mailman.utilities.i18n import make
from mailman.utilities.string import oneline



class Acknowledge:
    """Send an acknowledgment."""
    implements(IHandler)

    name = 'acknowledge'
    description = _("""Send an acknowledgment of a posting.""")

    def process(self, mlist, msg, msgdata):
        """See `IHandler`."""
        # Extract the sender's address and find them in the user database
        sender = msgdata.get('original_sender', msg.sender)
        member = mlist.members.get_member(sender)
        if member is None or not member.acknowledge_posts:
            # Either the sender is not a member, in which case we can't know
            # whether they want an acknowlegment or not, or they are a member
            # who definitely does not want an acknowlegment.
            return
        # Okay, they are a member that wants an acknowledgment of their post.
        # Give them their original subject.  BAW: do we want to use the
        # decoded header?
        original_subject = msgdata.get(
            'origsubj', msg.get('subject', _('(no subject)')))
        # Get the user's preferred language.
        language_manager = getUtility(ILanguageManager)
        language = (language_manager[msgdata['lang']]
                    if 'lang' in msgdata
                    else member.preferred_language)
        charset = language_manager[language.code].charset
        # Now get the acknowledgement template.
        display_name = mlist.display_name
        text = make('postack.txt',
                    mailing_list=mlist,
                    language=language.code,
                    wrap=False,
                    subject=oneline(original_subject, charset),
                    list_name=mlist.list_name,
                    display_name=display_name,
                    listinfo_url=mlist.script_url('listinfo'),
                    optionsurl=member.options_url,
                    )
        # Craft the outgoing message, with all headers and attributes
        # necessary for general delivery.  Then enqueue it to the outgoing
        # queue.
        subject = _('$display_name post acknowledgment')
        usermsg = UserNotification(sender, mlist.bounces_address,
                                   subject, text, language)
        usermsg.send(mlist)



# Copyright (C) 2012 by the Free Software Foundation, Inc.
#
# This file is part of GNU Mailman.
#
# GNU Mailman is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# GNU Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# GNU Mailman.  If not, see <http://www.gnu.org/licenses/>.

"""Template finder."""

from __future__ import absolute_import, print_function, unicode_literals

__metaclass__ = type
__all__ = [
    'TemplateFinder',
    ]


import os

from restish import http, resource

from mailman.config import config
from mailman.utilities.i18n import TemplateNotFoundError, find


# Use mimetypes.guess_all_extensions()?
EXTENSIONS = {
    'text/plain': '.txt',
    'text/html': '.html',
    }



class TemplateFinder(resource.Resource):
    """Template finder resource."""

    def __init__(self, mlist, template, language, content_type):
        self.mlist = mlist
        self.template = template
        self.language = language
        self.content_type = content_type

    @resource.GET()
    def find_template(self, request):
        # XXX We currently only support .txt and .html files.
        extension = EXTENSIONS.get(self.content_type)
        if extension is None:
            return http.not_found()
        template = self.template + extension
        fp = None
        try:
            try:
                path, fp = find(template, self.mlist, self.language)
            except TemplateNotFoundError:
                return http.not_found()
            else:
                return fp.read()
        finally:
            if fp is not None:
                fp.close()



# Copyright (C) 1998-2012 by the Free Software Foundation, Inc.
#
# This file is part of GNU Mailman.
#
# GNU Mailman is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# GNU Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# GNU Mailman.  If not, see <http://www.gnu.org/licenses/>.

import re
import sys
import time
import optparse

from mailman import MailList
from mailman import errors
from mailman.configuration import config
from mailman.core.i18n import _
from mailman.initialize import initialize
from mailman.utilities.string import wrap
from mailman.version import MAILMAN_VERSION


NL = '\n'
nonasciipat = re.compile(r'[\x80-\xff]')



def parseargs():
    parser = optparse.OptionParser(version=MAILMAN_VERSION,
                                   usage=_("""\
%prog [options] listname

Configure a list from a text file description, or dump a list's configuration
settings."""))
    parser.add_option('-i', '--inputfile',
                      metavar='FILENAME', default=None, type='string',
                      help=_("""\
Configure the list by assigning each module-global variable in the file to an
attribute on the mailing list object, then save the list.  The named file is
loaded with execfile() and must be legal Python code.  Any variable that isn't
already an attribute of the list object is ignored (a warning message is
printed).  See also the -c option.

A special variable named 'mlist' is put into the globals during the execfile,
which is bound to the actual MailList object.  This lets you do all manner of
bizarre thing to the list object, but BEWARE!  Using this can severely (and
possibly irreparably) damage your mailing list!

The may not be used with the -o option."""))
    parser.add_option('-o', '--outputfile',
                      metavar='FILENAME', default=None, type='string',
                      help=_("""\
Instead of configuring the list, print out a mailing list's configuration
variables in a format suitable for input using this script.  In this way, you
can easily capture the configuration settings for a particular list and
imprint those settings on another list.  FILENAME is the file to output the
settings to.  If FILENAME is `-', standard out is used.

This may not be used with the -i option."""))
    parser.add_option('-c', '--checkonly',
                      default=False, action='store_true', help=_("""\
With this option, the modified list is not actually changed.  This is only
useful with the -i option."""))
    parser.add_option('-v', '--verbose',
                      default=False, action='store_true', help=_("""\
Print the name of each attribute as it is being changed.  This is only useful
with the -i option."""))
    parser.add_option('-C', '--config',
                      help=_('Alternative configuration file to use'))
    opts, args = parser.parse_args()
    if len(args) > 1:
        parser.print_help()
        parser.error(_('Unexpected arguments'))
    if not args:
        parser.error(_('List name is required'))
    return parser, opts, args



def do_output(listname, outfile, parser):
    closep = False
    try:
        if outfile == '-':
            outfp = sys.stdout
        else:
            outfp = open(outfile, 'w')
            closep = True
        # Open the specified list unlocked, since we're only reading it.
        try:
            mlist = MailList.MailList(listname, lock=False)
        except errors.MMListError:
            parser.error(_('No such list: $listname'))
        # Preamble for the config info. PEP 263 charset and capture time.
        charset = mlist.preferred_language.charset
        # Set the system's default language.
        _.default = mlist.preferred_language.code
        if not charset:
            charset = 'us-ascii'
        when = time.ctime(time.time())
        print >> outfp, _('''\
# -*- python -*-
# -*- coding: $charset -*-
## "$listname" mailing list configuration settings
## captured on $when
''')
        # Get all the list config info.  All this stuff is accessible via the
        # web interface.
        for k in config.ADMIN_CATEGORIES:
            subcats = mlist.GetConfigSubCategories(k)
            if subcats is None:
                do_list_categories(mlist, k, None, outfp)
            else:
                for subcat in [t[0] for t in subcats]:
                    do_list_categories(mlist, k, subcat, outfp)
    finally:
        if closep:
            outfp.close()



def do_list_categories(mlist, k, subcat, outfp):
    info = mlist.GetConfigInfo(k, subcat)
    label, gui = mlist.GetConfigCategories()[k]
    if info is None:
        return
    charset = mlist.preferred_language.charset
    print >> outfp, '##', k.capitalize(), _('options')
    print >> outfp, '#'
    # First, massage the descripton text, which could have obnoxious
    # leading whitespace on second and subsequent lines due to
    # triple-quoted string nonsense in the source code.
    desc = NL.join([s.lstrip() for s in info[0].splitlines()])
    # Print out the category description
    desc = wrap(desc)
    for line in desc.splitlines():
        print >> outfp, '#', line
    print >> outfp
    for data in info[1:]:
        if not isinstance(data, tuple):
            continue
        varname = data[0]
        # Variable could be volatile
        if varname[0] == '_':
            continue
        vtype = data[1]
        # First, massage the descripton text, which could have
        # obnoxious leading whitespace on second and subsequent lines
        # due to triple-quoted string nonsense in the source code.
        desc = NL.join([s.lstrip() for s in data[-1].splitlines()])
        # Now strip out all HTML tags
        desc = re.sub('<.*?>', '', desc)
        # And convert &lt;/&gt; to <>
        desc = re.sub('&lt;', '<', desc)
        desc = re.sub('&gt;', '>', desc)
        # Print out the variable description.
        desc = wrap(desc)
        for line in desc.split('\n'):
            print >> outfp, '#', line
        # munge the value based on its type
        value = None
        if hasattr(gui, 'getValue'):
            value = gui.getValue(mlist, vtype, varname, data[2])
        if value is None and not varname.startswith('_'):
            value = getattr(mlist, varname)
        if vtype in (config.String, config.Text, config.FileUpload):
            print >> outfp, varname, '=',
            lines = value.splitlines()
            if not lines:
                print >> outfp, "''"
            elif len(lines) == 1:
                if charset != 'us-ascii' and nonasciipat.search(lines[0]):
                    # This is more readable for non-english list.
                    print >> outfp, '"' + lines[0].replace('"', '\\"') + '"'
                else:
                    print >> outfp, repr(lines[0])
            else:
                if charset == 'us-ascii' and nonasciipat.search(value):
                    # Normally, an english list should not have non-ascii char.
                    print >> outfp, repr(NL.join(lines))
                else:
                    outfp.write(' """')
                    outfp.write(NL.join(lines).replace('"', '\\"'))
                    outfp.write('"""\n')
        elif vtype in (config.Radio, config.Toggle):
            print >> outfp, '#'
            print >> outfp, '#', _('legal values are:')
            # TBD: This is disgusting, but it's special cased
            # everywhere else anyway...
            if varname == 'subscribe_policy' and \
                   not config.ALLOW_OPEN_SUBSCRIBE:
                i = 1
            else:
                i = 0
            for choice in data[2]:
                print >> outfp, '#   ', i, '= "%s"' % choice
                i += 1
            print >> outfp, varname, '=', repr(value)
        else:
            print >> outfp, varname, '=', repr(value)
        print >> outfp



def getPropertyMap(mlist):
    guibyprop = {}
    categories = mlist.GetConfigCategories()
    for category, (label, gui) in categories.items():
        if not hasattr(gui, 'GetConfigInfo'):
            continue
        subcats = mlist.GetConfigSubCategories(category)
        if subcats is None:
            subcats = [(None, None)]
        for subcat, sclabel in subcats:
            for element in gui.GetConfigInfo(mlist, category, subcat):
                if not isinstance(element, tuple):
                    continue
                propname = element[0]
                wtype = element[1]
                guibyprop[propname] = (gui, wtype)
    return guibyprop


class FakeDoc:
    # Fake the error reporting API for the htmlformat.Document class
    def addError(self, s, tag=None, *args):
        if tag:
            print >> sys.stderr, tag
        print >> sys.stderr, s % args

    def set_language(self, val):
        pass



def do_input(listname, infile, checkonly, verbose, parser):
    fakedoc = FakeDoc()
    # Open the specified list locked, unless checkonly is set
    try:
        mlist = MailList.MailList(listname, lock=not checkonly)
    except errors.MMListError as error:
        parser.error(_('No such list "$listname"\n$error'))
    savelist = False
    guibyprop = getPropertyMap(mlist)
    try:
        globals = {'mlist': mlist}
        # Any exception that occurs in execfile() will cause the list to not
        # be saved, but any other problems are not save-fatal.
        execfile(infile, globals)
        savelist = True
        for k, v in globals.items():
            if k in ('mlist', '__builtins__'):
                continue
            if not hasattr(mlist, k):
                print >> sys.stderr, _('attribute "$k" ignored')
                continue
            if verbose:
                print >> sys.stderr, _('attribute "$k" changed')
            missing = []
            gui, wtype = guibyprop.get(k, (missing, missing))
            if gui is missing:
                # This isn't an official property of the list, but that's
                # okay, we'll just restore it the old fashioned way
                print >> sys.stderr, _('Non-standard property restored: $k')
                setattr(mlist, k, v)
            else:
                # BAW: This uses non-public methods.  This logic taken from
                # the guts of GUIBase.handleForm().
                try:
                    validval = gui._getValidValue(mlist, k, wtype, v)
                except ValueError:
                    print >> sys.stderr, _('Invalid value for property: $k')
                except errors.EmailAddressError:
                    print >> sys.stderr, _(
                        'Bad email address for option $k: $v')
                else:
                    # BAW: Horrible hack, but then this is special cased
                    # everywhere anyway. :(  Privacy._setValue() knows that
                    # when ALLOW_OPEN_SUBSCRIBE is false, the web values are
                    # 0, 1, 2 but these really should be 1, 2, 3, so it adds
                    # one.  But we really do provide [0..3] so we need to undo
                    # the hack that _setValue adds. :( :(
                    if k == 'subscribe_policy' and \
                           not config.ALLOW_OPEN_SUBSCRIBE:
                        validval -= 1
                    # BAW: Another horrible hack.  This one is just too hard
                    # to fix in a principled way in Mailman 2.1
                    elif k == 'new_member_options':
                        # Because this is a Checkbox, _getValidValue()
                        # transforms the value into a list of one item.
                        validval = validval[0]
                        validval = [bitfield for bitfield, bitval
                                    in config.OPTINFO.items()
                                    if validval & bitval]
                    gui._setValue(mlist, k, validval, fakedoc)
            # BAW: when to do gui._postValidate()???
    finally:
        if savelist and not checkonly:
            mlist.Save()
        mlist.Unlock()



def main():
    parser, opts, args = parseargs()
    initialize(opts.config)
    listname = args[0]

    # Sanity check
    if opts.inputfile and opts.outputfile:
        parser.error(_('Only one of -i or -o is allowed'))
    if not opts.inputfile and not opts.outputfile:
        parser.error(_('One of -i or -o is required'))

    if opts.outputfile:
        do_output(listname, opts.outputfile, parser)
    else:
        do_input(listname, opts.inputfile, opts.checkonly,
                 opts.verbose, parser)



if __name__ == '__main__':
    main()



# Copyright (C) 2007-2012 by the Free Software Foundation, Inc.
#
# This file is part of GNU Mailman.
#
# GNU Mailman is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# GNU Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# GNU Mailman.  If not, see <http://www.gnu.org/licenses/>.

"""A user manager."""

from __future__ import absolute_import, unicode_literals

__metaclass__ = type
__all__ = [
    'UserManager',
    ]


from zope.interface import implements

from mailman.config import config
from mailman.interfaces.address import ExistingAddressError
from mailman.interfaces.usermanager import IUserManager
from mailman.model.address import Address
from mailman.model.member import Member
from mailman.model.preferences import Preferences
from mailman.model.user import User



class UserManager:
    implements(IUserManager)

    def create_user(self, email=None, display_name=None):
        """See `IUserManager`."""
        user = User(display_name, Preferences())
        if email:
            address = self.create_address(email, display_name)
            user.link(address)
        return user

    def delete_user(self, user):
        """See `IUserManager`."""
        config.db.store.remove(user)

    def get_user(self, email):
        """See `IUserManager`."""
        addresses = config.db.store.find(Address, email=email.lower())
        if addresses.count() == 0:
            return None
        return addresses.one().user

    def get_user_by_id(self, user_id):
        """See `IUserManager`."""
        users = config.db.store.find(User, _user_id=user_id)
        if users.count() == 0:
            return None
        return users.one()

    @property
    def users(self):
        """See `IUserManager`."""
        for user in config.db.store.find(User):
            yield user

    def create_address(self, email, display_name=None):
        """See `IUserManager`."""
        addresses = config.db.store.find(Address, email=email.lower())
        if addresses.count() == 1:
            found = addresses[0]
            raise ExistingAddressError(found.original_email)
        assert addresses.count() == 0, 'Unexpected results'
        if display_name is None:
            display_name = ''
        # It's okay not to lower case the 'email' argument because the
        # constructor will do the right thing.
        address = Address(email, display_name)
        address.preferences = Preferences()
        config.db.store.add(address)
        return address

    def delete_address(self, address):
        """See `IUserManager`."""
        # If there's a user controlling this address, it has to first be
        # unlinked before the address can be deleted.
        if address.user:
            address.user.unlink(address)
        config.db.store.remove(address)

    def get_address(self, email):
        """See `IUserManager`."""
        addresses = config.db.store.find(Address, email=email.lower())
        if addresses.count() == 0:
            return None
        return addresses.one()

    @property
    def addresses(self):
        """See `IUserManager`."""
        for address in config.db.store.find(Address):
            yield address

    @property
    def members(self):
        """See `IUserManager."""
        for member in config.db.store.find(Member):
                yield member



# Copyright (C) 2008-2012 by the Free Software Foundation, Inc.
#
# This file is part of GNU Mailman.
#
# GNU Mailman is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# GNU Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# GNU Mailman.  If not, see <http://www.gnu.org/licenses/>.

"""Interface for archiving schemes."""

from __future__ import absolute_import, unicode_literals

__metaclass__ = type
__all__ = [
    'IArchiver',
    ]


from zope.interface import Interface, Attribute



class IArchiver(Interface):
    """An interface to the archiver."""

    name = Attribute('The name of this archiver')

    def list_url(mlist):
        """Return the url to the top of the list's archive.

        :param mlist: The IMailingList object.
        :returns: The url string.
        """

    def permalink(mlist, msg):
        """Return the url to the message in the archive.

        This url points directly to the message in the archive.  This method
        only calculates the url, it does not actually archive the message.

        :param mlist: The IMailingList object.
        :param msg: The message object.
        :returns: The url string or None if the message's archive url cannot
            be calculated.
        """

    def archive_message(mlist, msg):
        """Send the message to the archiver.

        :param mlist: The IMailingList object.
        :param msg: The message object.
        :returns: The url string or None if the message's archive url cannot
            be calculated.
        """

    # XXX How to handle attachments?



# Copyright (C) 2012 by the Free Software Foundation, Inc.
#
# This file is part of GNU Mailman.
#
# GNU Mailman is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# GNU Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# GNU Mailman.  If not, see <http://www.gnu.org/licenses/>.

"""Tests for the LMTP server."""

from __future__ import absolute_import, print_function, unicode_literals

__metaclass__ = type
__all__ = [
    'TestLMTP',
    ]


import smtplib
import unittest

from mailman.app.lifecycle import create_list
from mailman.config import config
from mailman.testing.helpers import get_lmtp_client, get_queue_messages
from mailman.testing.layers import LMTPLayer



class TestLMTP(unittest.TestCase):
    """Test various aspects of the LMTP server."""

    layer = LMTPLayer

    def setUp(self):
        self._mlist = create_list('test@example.com')
        config.db.commit()
        self._lmtp = get_lmtp_client(quiet=True)
        self._lmtp.lhlo('remote.example.org')

    def tearDown(self):
        self._lmtp.close()

    def test_message_id_required(self):
        # The message is rejected if it does not have a Message-ID header.
        try:
            self._lmtp.sendmail('anne@example.com', ['test@example.com'], """\
From: anne@example.com
To: test@example.com
Subject: This has no Message-ID header

""")
        except smtplib.SMTPDataError as error:
            pass
        else:
            raise AssertionError('SMTPDataError expected')
        # LMTP returns a 550: Requested action not taken: mailbox unavailable
        # (e.g., mailbox not found, no access, or command rejected for policy
        # reasons)
        self.assertEqual(error.smtp_code, 550)
        self.assertEqual(error.smtp_error, 'No Message-ID header provided')

    def test_message_id_hash_is_added(self):
        self._lmtp.sendmail('anne@example.com', ['test@example.com'], """\
From: anne@example.com
To: test@example.com
Message-ID: <ant>
Subject: This has a Message-ID but no X-Message-ID-Hash

""")
        messages = get_queue_messages('in')
        self.assertEqual(len(messages), 1)
        self.assertEqual(messages[0].msg['x-message-id-hash'],
                         'MS6QLWERIJLGCRF44J7USBFDELMNT2BW')

    def test_original_message_id_hash_is_overwritten(self):
        self._lmtp.sendmail('anne@example.com', ['test@example.com'], """\
From: anne@example.com
To: test@example.com
Message-ID: <ant>
X-Message-ID-Hash: IGNOREME
Subject: This has a Message-ID but no X-Message-ID-Hash

""")
        messages = get_queue_messages('in')
        self.assertEqual(len(messages), 1)
        all_headers = messages[0].msg.get_all('x-message-id-hash')
        self.assertEqual(len(all_headers), 1)
        self.assertEqual(messages[0].msg['x-message-id-hash'],
                         'MS6QLWERIJLGCRF44J7USBFDELMNT2BW')



# Copyright (C) 2006-2012 by the Free Software Foundation, Inc.
#
# This file is part of GNU Mailman.
#
# GNU Mailman is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# GNU Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# GNU Mailman.  If not, see <http://www.gnu.org/licenses/>.

"""Model for mailing lists."""

from __future__ import absolute_import, unicode_literals

__metaclass__ = type
__all__ = [
    'MailingList',
    ]


import os
import string

from storm.locals import (
    And, Bool, DateTime, Float, Int, Pickle, Reference, Store, TimeDelta,
    Unicode)
from urlparse import urljoin
from zope.component import getUtility
from zope.interface import implements

from mailman.config import config
from mailman.database.model import Model
from mailman.database.types import Enum
from mailman.interfaces.action import Action, FilterAction
from mailman.interfaces.address import IAddress
from mailman.interfaces.autorespond import ResponseAction
from mailman.interfaces.bounce import UnrecognizedBounceDisposition
from mailman.interfaces.digests import DigestFrequency
from mailman.interfaces.domain import IDomainManager
from mailman.interfaces.languages import ILanguageManager
from mailman.interfaces.mailinglist import (
    IAcceptableAlias, IAcceptableAliasSet, IMailingList, Personalization,
    ReplyToMunging)
from mailman.interfaces.member import (
    AlreadySubscribedError, MemberRole, MissingPreferredAddressError)
from mailman.interfaces.mime import FilterType
from mailman.interfaces.nntp import NewsModeration
from mailman.interfaces.user import IUser
from mailman.model import roster
from mailman.model.digests import OneLastDigest
from mailman.model.member import Member
from mailman.model.mime import ContentFilter
from mailman.model.preferences import Preferences
from mailman.utilities.filesystem import makedirs
from mailman.utilities.string import expand


SPACE = ' '
UNDERSCORE = '_'



class MailingList(Model):
    implements(IMailingList)

    id = Int(primary=True)

    # XXX denotes attributes that should be part of the public interface but
    # are currently missing.

    # List identity
    list_name = Unicode()
    mail_host = Unicode()
    include_list_post_header = Bool()
    include_rfc2369_headers = Bool()
    advertised = Bool()
    anonymous_list = Bool()
    # Attributes not directly modifiable via the web u/i
    created_at = DateTime()
    admin_member_chunksize = Int()
    # Attributes which are directly modifiable via the web u/i.  The more
    # complicated attributes are currently stored as pickles, though that
    # will change as the schema and implementation is developed.
    next_request_id = Int()
    next_digest_number = Int()
    digest_last_sent_at = DateTime()
    volume = Int()
    last_post_at = DateTime()
    # Implicit destination.
    acceptable_aliases_id = Int()
    acceptable_alias = Reference(acceptable_aliases_id, 'AcceptableAlias.id')
    # Attributes which are directly modifiable via the web u/i.  The more
    # complicated attributes are currently stored as pickles, though that
    # will change as the schema and implementation is developed.
    accept_these_nonmembers = Pickle() # XXX
    admin_immed_notify = Bool()
    admin_notify_mchanges = Bool()
    administrivia = Bool()
    archive = Bool() # XXX
    archive_private = Bool() # XXX
    archive_volume_frequency = Int() # XXX
    # Automatic responses.
    autoresponse_grace_period = TimeDelta()
    autorespond_owner = Enum(ResponseAction)
    autoresponse_owner_text = Unicode()
    autorespond_postings = Enum(ResponseAction)
    autoresponse_postings_text = Unicode()
    autorespond_requests = Enum(ResponseAction)
    autoresponse_request_text = Unicode()
    # Content filters.
    filter_action = Enum(FilterAction)
    filter_content = Bool()
    collapse_alternatives = Bool()
    convert_html_to_plaintext = Bool()
    # Bounces.
    bounce_info_stale_after = TimeDelta() # XXX
    bounce_matching_headers = Unicode() # XXX
    bounce_notify_owner_on_disable = Bool() # XXX
    bounce_notify_owner_on_removal = Bool() # XXX
    bounce_score_threshold = Int() # XXX
    bounce_you_are_disabled_warnings = Int() # XXX
    bounce_you_are_disabled_warnings_interval = TimeDelta() # XXX
    forward_unrecognized_bounces_to = Enum(UnrecognizedBounceDisposition)
    process_bounces = Bool()
    # Miscellaneous
    default_member_action = Enum(Action)
    default_nonmember_action = Enum(Action)
    description = Unicode()
    digest_footer_uri = Unicode()
    digest_header_uri = Unicode()
    digest_is_default = Bool()
    digest_send_periodic = Bool()
    digest_size_threshold = Float()
    digest_volume_frequency = Enum(DigestFrequency)
    digestable = Bool()
    discard_these_nonmembers = Pickle()
    emergency = Bool()
    encode_ascii_prefixes = Bool()
    first_strip_reply_to = Bool()
    footer_uri = Unicode()
    forward_auto_discards = Bool()
    gateway_to_mail = Bool()
    gateway_to_news = Bool()
    generic_nonmember_action = Int()
    goodbye_message_uri = Unicode()
    header_matches = Pickle()
    header_uri = Unicode()
    hold_these_nonmembers = Pickle()
    info = Unicode()
    linked_newsgroup = Unicode()
    max_days_to_hold = Int()
    max_message_size = Int()
    max_num_recipients = Int()
    member_moderation_notice = Unicode()
    mime_is_default_digest = Bool()
    moderator_password = Unicode()
    new_member_options = Int()
    news_moderation = Enum(NewsModeration)
    news_prefix_subject_too = Bool()
    nntp_host = Unicode()
    nondigestable = Bool()
    nonmember_rejection_notice = Unicode()
    obscure_addresses = Bool()
    owner_chain = Unicode()
    owner_pipeline = Unicode()
    personalize = Enum(Personalization)
    post_id = Int()
    posting_chain = Unicode()
    posting_pipeline = Unicode()
    _preferred_language = Unicode(name='preferred_language')
    private_roster = Bool()
    display_name = Unicode()
    reject_these_nonmembers = Pickle()
    reply_goes_to_list = Enum(ReplyToMunging)
    reply_to_address = Unicode()
    require_explicit_destination = Bool()
    respond_to_post_requests = Bool()
    scrub_nondigest = Bool()
    send_goodbye_message = Bool()
    send_reminders = Bool()
    send_welcome_message = Bool()
    subject_prefix = Unicode()
    subscribe_auto_approval = Pickle()
    subscribe_policy = Int()
    topics = Pickle()
    topics_bodylines_limit = Int()
    topics_enabled = Bool()
    unsubscribe_policy = Int()
    welcome_message_uri = Unicode()

    def __init__(self, fqdn_listname):
        super(MailingList, self).__init__()
        listname, at, hostname = fqdn_listname.partition('@')
        assert hostname, 'Bad list name: {0}'.format(fqdn_listname)
        self.list_name = listname
        self.mail_host = hostname
        # For the pending database
        self.next_request_id = 1
        # We need to set up the rosters.  Normally, this method will get
        # called when the MailingList object is loaded from the database, but
        # that's not the case when the constructor is called.  So, set up the
        # rosters explicitly.
        self.__storm_loaded__()
        self.personalize = Personalization.none
        self.display_name = string.capwords(
            SPACE.join(listname.split(UNDERSCORE)))
        makedirs(self.data_path)

    def __storm_loaded__(self):
        self.owners = roster.OwnerRoster(self)
        self.moderators = roster.ModeratorRoster(self)
        self.administrators = roster.AdministratorRoster(self)
        self.members = roster.MemberRoster(self)
        self.regular_members = roster.RegularMemberRoster(self)
        self.digest_members = roster.DigestMemberRoster(self)
        self.subscribers = roster.Subscribers(self)
        self.nonmembers = roster.NonmemberRoster(self)

    def __repr__(self):
        return '<mailing list "{0}" at {1:#x}>'.format(
            self.fqdn_listname, id(self))

    @property
    def fqdn_listname(self):
        """See `IMailingList`."""
        return '{0}@{1}'.format(self.list_name, self.mail_host)

    @property
    def domain(self):
        """See `IMailingList`."""
        return getUtility(IDomainManager)[self.mail_host]

    @property
    def scheme(self):
        """See `IMailingList`."""
        return self.domain.scheme

    @property
    def web_host(self):
        """See `IMailingList`."""
        return self.domain.url_host

    def script_url(self, target, context=None):
        """See `IMailingList`."""
        # XXX Handle the case for when context is not None; those would be
        # relative URLs.
        return urljoin(self.domain.base_url, target + '/' + self.fqdn_listname)

    @property
    def data_path(self):
        """See `IMailingList`."""
        return os.path.join(config.LIST_DATA_DIR, self.fqdn_listname)

    # IMailingListAddresses

    @property
    def posting_address(self):
        """See `IMailingList`."""
        return self.fqdn_listname

    @property
    def no_reply_address(self):
        """See `IMailingList`."""
        return '{0}@{1}'.format(config.mailman.noreply_address, self.mail_host)

    @property
    def owner_address(self):
        """See `IMailingList`."""
        return '{0}-owner@{1}'.format(self.list_name, self.mail_host)

    @property
    def request_address(self):
        """See `IMailingList`."""
        return '{0}-request@{1}'.format(self.list_name, self.mail_host)

    @property
    def bounces_address(self):
        """See `IMailingList`."""
        return '{0}-bounces@{1}'.format(self.list_name, self.mail_host)

    @property
    def join_address(self):
        """See `IMailingList`."""
        return '{0}-join@{1}'.format(self.list_name, self.mail_host)

    @property
    def leave_address(self):
        """See `IMailingList`."""
        return '{0}-leave@{1}'.format(self.list_name, self.mail_host)

    @property
    def subscribe_address(self):
        """See `IMailingList`."""
        return '{0}-subscribe@{1}'.format(self.list_name, self.mail_host)

    @property
    def unsubscribe_address(self):
        """See `IMailingList`."""
        return '{0}-unsubscribe@{1}'.format(self.list_name, self.mail_host)

    def confirm_address(self, cookie):
        """See `IMailingList`."""
        local_part = expand(config.mta.verp_confirm_format, dict(
            address = '{0}-confirm'.format(self.list_name),
            cookie  = cookie))
        return '{0}@{1}'.format(local_part, self.mail_host)

    @property
    def preferred_language(self):
        """See `IMailingList`."""
        return getUtility(ILanguageManager)[self._preferred_language]

    @preferred_language.setter
    def preferred_language(self, language):
        """See `IMailingList`."""
        # Accept both a language code and a `Language` instance.
        try:
            self._preferred_language = language.code
        except AttributeError:
            self._preferred_language = language

    def send_one_last_digest_to(self, address, delivery_mode):
        """See `IMailingList`."""
        digest = OneLastDigest(self, address, delivery_mode)
        Store.of(self).add(digest)

    @property
    def last_digest_recipients(self):
        """See `IMailingList`."""
        results = Store.of(self).find(
            OneLastDigest,
            OneLastDigest.mailing_list == self)
        recipients = [(digest.address, digest.delivery_mode)
                      for digest in results]
        results.remove()
        return recipients

    @property
    def filter_types(self):
        """See `IMailingList`."""
        results = Store.of(self).find(
            ContentFilter,
            And(ContentFilter.mailing_list == self,
                ContentFilter.filter_type == FilterType.filter_mime))
        for content_filter in results:
            yield content_filter.filter_pattern

    @filter_types.setter
    def filter_types(self, sequence):
        """See `IMailingList`."""
        # First, delete all existing MIME type filter patterns.
        store = Store.of(self)
        results = store.find(
            ContentFilter,
            And(ContentFilter.mailing_list == self,
                ContentFilter.filter_type == FilterType.filter_mime))
        results.remove()
        # Now add all the new filter types.
        for mime_type in sequence:
            content_filter = ContentFilter(
                self, mime_type, FilterType.filter_mime)
            store.add(content_filter)

    @property
    def pass_types(self):
        """See `IMailingList`."""
        results = Store.of(self).find(
            ContentFilter,
            And(ContentFilter.mailing_list == self,
                ContentFilter.filter_type == FilterType.pass_mime))
        for content_filter in results:
            yield content_filter.filter_pattern

    @pass_types.setter
    def pass_types(self, sequence):
        """See `IMailingList`."""
        # First, delete all existing MIME type pass patterns.
        store = Store.of(self)
        results = store.find(
            ContentFilter,
            And(ContentFilter.mailing_list == self,
                ContentFilter.filter_type == FilterType.pass_mime))
        results.remove()
        # Now add all the new filter types.
        for mime_type in sequence:
            content_filter = ContentFilter(
                self, mime_type, FilterType.pass_mime)
            store.add(content_filter)

    @property
    def filter_extensions(self):
        """See `IMailingList`."""
        results = Store.of(self).find(
            ContentFilter,
            And(ContentFilter.mailing_list == self,
                ContentFilter.filter_type == FilterType.filter_extension))
        for content_filter in results:
            yield content_filter.filter_pattern

    @filter_extensions.setter
    def filter_extensions(self, sequence):
        """See `IMailingList`."""
        # First, delete all existing file extensions filter patterns.
        store = Store.of(self)
        results = store.find(
            ContentFilter,
            And(ContentFilter.mailing_list == self,
                ContentFilter.filter_type == FilterType.filter_extension))
        results.remove()
        # Now add all the new filter types.
        for mime_type in sequence:
            content_filter = ContentFilter(
                self, mime_type, FilterType.filter_extension)
            store.add(content_filter)

    @property
    def pass_extensions(self):
        """See `IMailingList`."""
        results = Store.of(self).find(
            ContentFilter,
            And(ContentFilter.mailing_list == self,
                ContentFilter.filter_type == FilterType.pass_extension))
        for content_filter in results:
            yield content_filter.pass_pattern

    @pass_extensions.setter
    def pass_extensions(self, sequence):
        """See `IMailingList`."""
        # First, delete all existing file extensions pass patterns.
        store = Store.of(self)
        results = store.find(
            ContentFilter,
            And(ContentFilter.mailing_list == self,
                ContentFilter.filter_type == FilterType.pass_extension))
        results.remove()
        # Now add all the new filter types.
        for mime_type in sequence:
            content_filter = ContentFilter(
                self, mime_type, FilterType.pass_extension)
            store.add(content_filter)

    def get_roster(self, role):
        """See `IMailingList`."""
        if role is MemberRole.member:
            return self.members
        elif role is MemberRole.owner:
            return self.owners
        elif role is MemberRole.moderator:
            return self.moderators
        else:
            raise TypeError(
                'Undefined MemberRole: {0}'.format(role))

    def subscribe(self, subscriber, role=MemberRole.member):
        """See `IMailingList`."""
        store = Store.of(self)
        if IAddress.providedBy(subscriber):
            member = store.find(
                Member,
                Member.role == role,
                Member.mailing_list == self.fqdn_listname,
                Member._address == subscriber).one()
            if member:
                raise AlreadySubscribedError(
                    self.fqdn_listname, subscriber.email, role)
        elif IUser.providedBy(subscriber):
            if subscriber.preferred_address is None:
                raise MissingPreferredAddressError(subscriber)
            member = store.find(
                Member,
                Member.role == role,
                Member.mailing_list == self.fqdn_listname,
                Member._user == subscriber).one()
            if member:
                raise AlreadySubscribedError(
                    self.fqdn_listname, subscriber, role)
        else:
            raise ValueError('subscriber must be an address or user')
        member = Member(role=role,
                        mailing_list=self.fqdn_listname,
                        subscriber=subscriber)
        member.preferences = Preferences()
        store.add(member)
        return member



class AcceptableAlias(Model):
    implements(IAcceptableAlias)

    id = Int(primary=True)

    mailing_list_id = Int()
    mailing_list = Reference(mailing_list_id, MailingList.id)

    alias = Unicode()

    def __init__(self, mailing_list, alias):
        self.mailing_list = mailing_list
        self.alias = alias


class AcceptableAliasSet:
    implements(IAcceptableAliasSet)

    def __init__(self, mailing_list):
        self._mailing_list = mailing_list

    def clear(self):
        """See `IAcceptableAliasSet`."""
        Store.of(self._mailing_list).find(
            AcceptableAlias,
            AcceptableAlias.mailing_list == self._mailing_list).remove()

    def add(self, alias):
        if not (alias.startswith('^') or '@' in alias):
            raise ValueError(alias)
        alias = AcceptableAlias(self._mailing_list, alias.lower())
        Store.of(self._mailing_list).add(alias)

    def remove(self, alias):
        Store.of(self._mailing_list).find(
            AcceptableAlias,
            And(AcceptableAlias.mailing_list == self._mailing_list,
                AcceptableAlias.alias == alias.lower())).remove()

    @property
    def aliases(self):
        aliases = Store.of(self._mailing_list).find(
            AcceptableAlias,
            AcceptableAlias.mailing_list == self._mailing_list)
        for alias in aliases:
            yield alias.alias



# Copyright (C) 2011-2012 by the Free Software Foundation, Inc.
#
# This file is part of GNU Mailman.
#
# GNU Mailman is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# GNU Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# GNU Mailman.  If not, see <http://www.gnu.org/licenses/>.

"""REST for addresses."""

from __future__ import absolute_import, unicode_literals

__metaclass__ = type
__all__ = [
    'AllAddresses',
    'AnAddress',
    'UserAddresses',
    ]


from operator import attrgetter
from restish import http, resource
from zope.component import getUtility

from mailman.rest.helpers import CollectionMixin, etag, path_to
from mailman.rest.members import MemberCollection
from mailman.rest.preferences import Preferences
from mailman.interfaces.usermanager import IUserManager



class _AddressBase(resource.Resource, CollectionMixin):
    """Shared base class for address representations."""

    def _resource_as_dict(self, address):
        """See `CollectionMixin`."""
        # The canonical url for an address is its lower-cased version,
        # although it can be looked up with either its original or lower-cased
        # email address.
        representation = dict(
            email=address.email,
            original_email=address.original_email,
            registered_on=address.registered_on,
            self_link=path_to('addresses/{0}'.format(address.email)),
            )
        # Add optional attributes.  These can be None or the empty string.
        if address.display_name:
            representation['display_name'] = address.display_name
        if address.verified_on:
            representation['verified_on'] = address.verified_on
        return representation

    def _get_collection(self, request):
        """See `CollectionMixin`."""
        return list(getUtility(IUserManager).addresses)



class AllAddresses(_AddressBase):
    """The addresses."""

    @resource.GET()
    def collection(self, request):
        """/addresses"""
        resource = self._make_collection(request)
        return http.ok([], etag(resource))



class AnAddress(_AddressBase):
    """An address."""

    def __init__(self, email):
        """Get an address by either its original or lower-cased email.

        :param email: The email address of the `IAddress`.
        :type email: string
        """
        self._address = getUtility(IUserManager).get_address(email)

    @resource.GET()
    def address(self, request):
        """Return a single address."""
        if self._address is None:
            return http.not_found()
        return http.ok([], self._resource_as_json(self._address))

    @resource.child()
    def memberships(self, request, segments):
        """/addresses/<email>/memberships"""
        if len(segments) != 0:
            return http.bad_request()
        if self._address is None:
            return http.not_found()
        return AddressMemberships(self._address)

    @resource.child()
    def preferences(self, request, segments):
        """/addresses/<email>/preferences"""
        if len(segments) != 0:
            return http.bad_request()
        if self._address is None:
            return http.not_found()
        child = Preferences(
            self._address.preferences,
            'addresses/{0}'.format(self._address.email))
        return child, []



class UserAddresses(_AddressBase):
    """The addresses of a user."""

    def __init__(self, user):
        self._user = user
        super(UserAddresses, self).__init__()

    def _get_collection(self, request):
        """See `CollectionMixin`."""
        return sorted(self._user.addresses,
                      key=attrgetter('original_email'))

    @resource.GET()
    def collection(self, request):
        """/addresses"""
        resource = self._make_collection(request)
        return http.ok([], etag(resource))



def membership_key(member):
    # Sort first by mailing list, then by address, then by role.
    return member.mailing_list, member.address.email, int(member.role)


class AddressMemberships(MemberCollection):
    """All the memberships of a particular email address."""

    def __init__(self, address):
        super(AddressMemberships, self).__init__()
        self._address = address

    def _get_collection(self, request):
        """See `CollectionMixin`."""
        # XXX Improve this by implementing a .memberships attribute on
        # IAddress, similar to the way IUser does it.
        #
        # Start by getting the IUser that controls this address.  For now, if
        # the address is not controlled by a user, return the empty set.
        # Later when we address the XXX comment, it will return some
        # memberships.  But really, it should not be legal to subscribe an
        # address to a mailing list that isn't controlled by a user -- maybe!
        user = getUtility(IUserManager).get_user(self._address.email)
        if user is None:
            return []
        return sorted((member for member in user.memberships.members
                       if member.address == self._address),
                      key=membership_key)



# Copyright (C) 2010-2012 by the Free Software Foundation, Inc.
#
# This file is part of GNU Mailman.
#
# GNU Mailman is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# GNU Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# GNU Mailman.  If not, see <http://www.gnu.org/licenses/>.

"""Mailing list configuration via REST API."""

from __future__ import absolute_import, unicode_literals

__metaclass__ = type
__all__ = [
    'ListConfiguration',
    ]


from lazr.config import as_boolean, as_timedelta
from restish import http, resource

from mailman.config import config
from mailman.interfaces.action import Action
from mailman.interfaces.autorespond import ResponseAction
from mailman.interfaces.mailinglist import IAcceptableAliasSet, ReplyToMunging
from mailman.rest.helpers import PATCH, etag, no_content
from mailman.rest.validator import Validator, enum_validator



class GetterSetter:
    """Get and set attributes on mailing lists.

    Most attributes are fairly simple - a getattr() or setattr() on the
    mailing list does the trick, with the appropriate encoding or decoding on
    the way in and out.  Encoding doesn't happen here though; the standard
    JSON library handles most types, but see ExtendedEncoder in
    mailman.rest.helpers for additional support.

    Others are more complicated since they aren't kept in the model as direct
    columns in the database.  These will use subclasses of this base class.
    Read-only attributes will have a decoder which always raises ValueError.
    """

    def __init__(self, decoder=None):
        """Create a getter/setter for a specific list attribute.

        :param decoder: The callable for decoding a web request value string
            into the specific data type needed by the `IMailingList`
            attribute.  Use None to indicate a read-only attribute.  The
            callable should raise ValueError when the web request value cannot
            be converted.
        :type decoder: callable
        """
        self.decoder = decoder

    def get(self, mlist, attribute):
        """Return the named mailing list attribute value.

        :param mlist: The mailing list.
        :type mlist: `IMailingList`
        :param attribute: The attribute name.
        :type attribute: string
        :return: The attribute value, ready for JSON encoding.
        :rtype: object
        """
        return getattr(mlist, attribute)

    def put(self, mlist, attribute, value):
        """Set the named mailing list attribute value.

        :param mlist: The mailing list.
        :type mlist: `IMailingList`
        :param attribute: The attribute name.
        :type attribute: string
        :param value: The new value for the attribute.
        :type request_value: object
        """
        setattr(mlist, attribute, value)

    def __call__(self, value):
        """Convert the value to its internal format.

        :param value: The web request value to convert.
        :type value: string
        :return: The converted value.
        :rtype: object
        """
        if self.decoder is None:
            return value
        return self.decoder(value)


class AcceptableAliases(GetterSetter):
    """Resource for the acceptable aliases of a mailing list."""

    def get(self, mlist, attribute):
        """Return the mailing list's acceptable aliases."""
        assert attribute == 'acceptable_aliases', (
            'Unexpected attribute: {0}'.format(attribute))
        aliases = IAcceptableAliasSet(mlist)
        return sorted(aliases.aliases)

    def put(self, mlist, attribute, value):
        """Change the acceptable aliases.

        Because this is a PUT operation, all previous aliases are cleared
        first.  Thus, this is an overwrite.  The keys in the request are
        ignored.
        """
        assert attribute == 'acceptable_aliases', (
            'Unexpected attribute: {0}'.format(attribute))
        alias_set = IAcceptableAliasSet(mlist)
        alias_set.clear()
        for alias in value:
            alias_set.add(unicode(alias))



# Additional validators for converting from web request strings to internal
# data types.  See below for details.

def pipeline_validator(pipeline_name):
    """Convert the pipeline name to a string, but only if it's known."""
    if pipeline_name in config.pipelines:
        return unicode(pipeline_name)
    raise ValueError('Unknown pipeline: {0}'.format(pipeline_name))


def list_of_unicode(values):
    """Turn a list of things into a list of unicodes."""
    return [unicode(value) for value in values]



# This is the list of IMailingList attributes that are exposed through the
# REST API.  The values of the keys are the GetterSetter instance holding the
# decoder used to convert the web request string to an internally valid value.
# The instance also contains the get() and put() methods used to retrieve and
# set the attribute values.  Its .decoder attribute will be None for read-only
# attributes.
#
# The decoder must either return the internal value or raise a ValueError if
# the conversion failed (e.g. trying to turn 'Nope' into a boolean).
#
# Many internal value types can be automatically JSON encoded, but see
# mailman.rest.helpers.ExtendedEncoder for specializations of certain types
# (e.g. datetimes, timedeltas, enums).

ATTRIBUTES = dict(
    acceptable_aliases=AcceptableAliases(list_of_unicode),
    admin_immed_notify=GetterSetter(as_boolean),
    admin_notify_mchanges=GetterSetter(as_boolean),
    administrivia=GetterSetter(as_boolean),
    advertised=GetterSetter(as_boolean),
    anonymous_list=GetterSetter(as_boolean),
    autorespond_owner=GetterSetter(enum_validator(ResponseAction)),
    autorespond_postings=GetterSetter(enum_validator(ResponseAction)),
    autorespond_requests=GetterSetter(enum_validator(ResponseAction)),
    autoresponse_grace_period=GetterSetter(as_timedelta),
    autoresponse_owner_text=GetterSetter(unicode),
    autoresponse_postings_text=GetterSetter(unicode),
    autoresponse_request_text=GetterSetter(unicode),
    bounces_address=GetterSetter(None),
    collapse_alternatives=GetterSetter(as_boolean),
    convert_html_to_plaintext=GetterSetter(as_boolean),
    created_at=GetterSetter(None),
    default_member_action=GetterSetter(enum_validator(Action)),
    default_nonmember_action=GetterSetter(enum_validator(Action)),
    description=GetterSetter(unicode),
    digest_last_sent_at=GetterSetter(None),
    digest_size_threshold=GetterSetter(float),
    filter_content=GetterSetter(as_boolean),
    fqdn_listname=GetterSetter(None),
    generic_nonmember_action=GetterSetter(int),
    mail_host=GetterSetter(None),
    include_list_post_header=GetterSetter(as_boolean),
    include_rfc2369_headers=GetterSetter(as_boolean),
    join_address=GetterSetter(None),
    last_post_at=GetterSetter(None),
    leave_address=GetterSetter(None),
    list_name=GetterSetter(None),
    next_digest_number=GetterSetter(None),
    no_reply_address=GetterSetter(None),
    owner_address=GetterSetter(None),
    post_id=GetterSetter(None),
    posting_address=GetterSetter(None),
    posting_pipeline=GetterSetter(pipeline_validator),
    display_name=GetterSetter(unicode),
    reply_goes_to_list=GetterSetter(enum_validator(ReplyToMunging)),
    request_address=GetterSetter(None),
    scheme=GetterSetter(None),
    send_welcome_message=GetterSetter(as_boolean),
    volume=GetterSetter(None),
    web_host=GetterSetter(None),
    welcome_message_uri=GetterSetter(unicode),
    )


VALIDATORS = ATTRIBUTES.copy()
for attribute, gettersetter in VALIDATORS.items():
    if gettersetter.decoder is None:
        del VALIDATORS[attribute]



class ListConfiguration(resource.Resource):
    """A mailing list configuration resource."""

    def __init__(self, mailing_list, attribute):
        self._mlist = mailing_list
        self._attribute = attribute

    @resource.GET()
    def get_configuration(self, request):
        """Get a mailing list configuration."""
        resource = {}
        if self._attribute is None:
            # Return all readable attributes.
            for attribute in ATTRIBUTES:
                value = ATTRIBUTES[attribute].get(self._mlist, attribute)
                resource[attribute] = value
        elif self._attribute not in ATTRIBUTES:
            return http.bad_request(
                [], b'Unknown attribute: {0}'.format(self._attribute))
        else:
            attribute = self._attribute
            value = ATTRIBUTES[attribute].get(self._mlist, attribute)
            resource[attribute] = value
        return http.ok([], etag(resource))

    # XXX 2010-09-01 barry: Refactor {put,patch}_configuration() for common
    # code paths.

    def _set_writable_attributes(self, validator, request):
        """Common code for setting all attributes given in the request.

        Returns an HTTP 400 when a request tries to write to a read-only
        attribute.
        """
        converted = validator(request)
        for key, value in converted.items():
            ATTRIBUTES[key].put(self._mlist, key, value)

    @resource.PUT()
    def put_configuration(self, request):
        """Set a mailing list configuration."""
        attribute = self._attribute
        if attribute is None:
            validator = Validator(**VALIDATORS)
            try:
                self._set_writable_attributes(validator, request)
            except ValueError as error:
                return http.bad_request([], str(error))
        elif attribute not in ATTRIBUTES:
            return http.bad_request(
                [], b'Unknown attribute: {0}'.format(attribute))
        elif ATTRIBUTES[attribute].decoder is None:
            return http.bad_request(
                [], b'Read-only attribute: {0}'.format(attribute))
        else:
            validator = Validator(**{attribute: VALIDATORS[attribute]})
            try:
                self._set_writable_attributes(validator, request)
            except ValueError as error:
                return http.bad_request([], str(error))
        return no_content()

    @PATCH()
    def patch_configuration(self, request):
        """Patch the configuration (i.e. partial update)."""
        # Validate only the partial subset of attributes given in the request.
        validationators = {}
        for attribute in request.PATCH:
            if attribute not in ATTRIBUTES:
                return http.bad_request(
                    [], b'Unknown attribute: {0}'.format(attribute))
            elif ATTRIBUTES[attribute].decoder is None:
                return http.bad_request(
                    [], b'Read-only attribute: {0}'.format(attribute))
            else:
                validationators[attribute] = VALIDATORS[attribute]
        validator = Validator(**validationators)
        try:
            self._set_writable_attributes(validator, request)
        except ValueError as error:
            return http.bad_request([], str(error))
        return no_content()



# Copyright (C) 2007-2012 by the Free Software Foundation, Inc.
#
# This file is part of GNU Mailman.
#
# GNU Mailman is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# GNU Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# GNU Mailman.  If not, see <http://www.gnu.org/licenses/>.

"""Membership related rules."""

from __future__ import absolute_import, unicode_literals

__metaclass__ = type
__all__ = [
    'MemberModeration',
    'NonmemberModeration',
    ]


from zope.component import getUtility
from zope.interface import implements

from mailman.core.i18n import _
from mailman.interfaces.action import Action
from mailman.interfaces.member import MemberRole
from mailman.interfaces.rules import IRule
from mailman.interfaces.usermanager import IUserManager



class MemberModeration:
    """The member moderation rule."""
    implements(IRule)

    name = 'member-moderation'
    description = _('Match messages sent by moderated members.')
    record = True

    def check(self, mlist, msg, msgdata):
        """See `IRule`."""
        for sender in msg.senders:
            member = mlist.members.get_member(sender)
            action = (None if member is None
                      else member.moderation_action)
            if action is Action.defer:
                # The regular moderation rules apply.
                return False
            elif action is not None:
                # We must stringify the moderation action so that it can be
                # stored in the pending request table.
                msgdata['moderation_action'] = action.name
                msgdata['moderation_sender'] = sender
                return True
        # The sender is not a member so this rule does not match.
        return False



class NonmemberModeration:
    """The nonmember moderation rule."""
    implements(IRule)

    name = 'nonmember-moderation'
    description = _('Match messages sent by nonmembers.')
    record = True

    def check(self, mlist, msg, msgdata):
        """See `IRule`."""
        user_manager = getUtility(IUserManager)
        # First ensure that all senders are already either members or
        # nonmembers.  If they are not subscribed in some role to the mailing
        # list, make them nonmembers.
        for sender in msg.senders:
            if (mlist.members.get_member(sender) is None and
                mlist.nonmembers.get_member(sender) is None):
                # The address is neither a member nor nonmember.
                address = user_manager.get_address(sender)
                assert address is not None, (
                    'Posting address is not registered: {0}'.format(sender))
                mlist.subscribe(address, MemberRole.nonmember)
        # Do nonmember moderation check.
        for sender in msg.senders:
            nonmember = mlist.nonmembers.get_member(sender)
            action = (None if nonmember is None
                      else nonmember.moderation_action)
            if action is Action.defer:
                # The regular moderation rules apply.
                return False
            elif action is not None:
                # We must stringify the moderation action so that it can be
                # stored in the pending request table.
                msgdata['moderation_action'] = action.name
                msgdata['moderation_sender'] = sender
                return True
        # The sender must be a member, so this rule does not match.
        return False



# Copyright (C) 2011-2012 by the Free Software Foundation, Inc.
#
# This file is part of GNU Mailman.
#
# GNU Mailman is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# GNU Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# GNU Mailman.  If not, see <http://www.gnu.org/licenses/>.

"""Unique ID generation.

Use these functions to create unique ids rather than inlining calls to hashlib
and whatnot.  These are better instrumented for testing purposes.
"""

from __future__ import absolute_import, unicode_literals

__metaclass__ = type
__all__ = [
    'UniqueIDFactory',
    'factory',
    ]


import os
import uuid
import errno

from flufl.lock import Lock

from mailman.config import config
from mailman.model.uid import UID
from mailman.testing import layers



class UniqueIDFactory:
    """A factory for unique ids."""

    def __init__(self, context=None):
        # We can't call reset() when the factory is created below, because
        # config.VAR_DIR will not be set at that time.  So initialize it at
        # the first use.
        self._uid_file = None
        self._lock_file = None
        self._lockobj = None
        self._context = context
        layers.MockAndMonkeyLayer.register_reset(self.reset)

    @property
    def _lock(self):
        if self._lockobj is None:
            # These will get automatically cleaned up by the test
            # infrastructure.
            self._uid_file = os.path.join(config.VAR_DIR, '.uid')
            if self._context:
                self._uid_file += '.' + self._context
            self._lock_file = self._uid_file + '.lock'
            self._lockobj = Lock(self._lock_file)
        return self._lockobj

    def new_uid(self):
        """Return a new UID.

        :return: The new uid
        :rtype: int
        """
        if layers.is_testing():
            # When in testing mode we want to produce predictable id, but we
            # need to coordinate this among separate processes.  We could use
            # the database, but I don't want to add schema just to handle this
            # case, and besides transactions could get aborted, causing some
            # ids to be recycled.  So we'll use a data file with a lock.  This
            # may still not be ideal due to race conditions, but I think the
            # tests will be serialized enough (and the ids reset between
            # tests) that it will not be a problem.  Maybe.
            return self._next_uid()
        while True:
            uid = uuid.uuid4()
            try:
                UID.record(uid)
            except ValueError:
                pass
            else:
                return uid

    def _next_uid(self):
        with self._lock:
            try:
                with open(self._uid_file) as fp:
                    uid = int(fp.read().strip())
                    next_uid = uid + 1
                with open(self._uid_file, 'w') as fp:
                    fp.write(str(next_uid))
                return uuid.UUID(int=uid)
            except IOError as error:
                if error.errno != errno.ENOENT:
                    raise
                with open(self._uid_file, 'w') as fp:
                    fp.write('2')
                return uuid.UUID(int=1)

    def reset(self):
        with self._lock:
            with open(self._uid_file, 'w') as fp:
                fp.write('1')



factory = UniqueIDFactory()



# Copyright (C) 2009-2012 by the Free Software Foundation, Inc.
#
# This file is part of GNU Mailman.
#
# GNU Mailman is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# GNU Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# GNU Mailman.  If not, see <http://www.gnu.org/licenses/>.

"""Module stuff."""

from __future__ import absolute_import, unicode_literals

__metaclass__ = type
__all__ = [
    'Confirm',
    ]


from zope.component import getUtility
from zope.interface import implements

from mailman.core.i18n import _
from mailman.interfaces.command import ContinueProcessing, IEmailCommand
from mailman.interfaces.registrar import IRegistrar



class Confirm:
    """The email 'confirm' command."""

    implements(IEmailCommand)

    name = 'confirm'
    argument_description = 'token'
    description = _('Confirm a subscription request.')
    short_description = description

    def process(self, mlist, msg, msgdata, arguments, results):
        """See `IEmailCommand`."""
        # The token must be in the arguments.
        if len(arguments) == 0:
            print >> results, _('No confirmation token found')
            return ContinueProcessing.no
        # Make sure we don't try to confirm the same token more than once.
        token = arguments[0]
        tokens = getattr(results, 'confirms', set())
        if token in tokens:
            # Do not try to confirm this one again.
            return ContinueProcessing.yes
        tokens.add(token)
        results.confirms = tokens
        succeeded = getUtility(IRegistrar).confirm(token)
        if succeeded:
            print >> results, _('Confirmed')
            return ContinueProcessing.yes
        print >> results, _('Confirmation token did not match')
        return ContinueProcessing.no



# Copyright (C) 2006-2012 by the Free Software Foundation, Inc.
#
# This file is part of GNU Mailman.
#
# GNU Mailman is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# GNU Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# GNU Mailman.  If not, see <http://www.gnu.org/licenses/>.

import os
import sys

from cPickle import load

from mailman.config import config
from mailman.core.i18n import _
from mailman.options import Options



class ScriptOptions(Options):
    usage = _("""
%%prog [options] qfiles ...

Show the contents of one or more Mailman queue files.""")

    def add_options(self):
        super(ScriptOptions, self).add_options()
        self.parser.add_option(
            '-q', '--quiet',
            default=False, action='store_true',
            help=_("Don't print 'helpful' message delimiters."))
        self.parser.add_option(
            '-s', '--summary',
            default=False, action='store_true',
            help=_('Show a summary of queue files.'))



def main():
    options = ScriptOptions()
    options.initialize()

    if options.options.summary:
        queue_totals = {}
        files_by_queue = {}
        for switchboard in config.switchboards.values():
            total = 0
            file_mappings = {}
            for filename in os.listdir(switchboard.queue_directory):
                base, ext = os.path.splitext(filename)
                file_mappings[ext] = file_mappings.get(ext, 0) + 1
                total += 1
            files_by_queue[switchboard.queue_directory] = file_mappings
            queue_totals[switchboard.queue_directory] = total
        # Sort by queue name.
        for queue_directory in sorted(files_by_queue):
            total = queue_totals[queue_directory]
            print queue_directory
            print _('\tfile count: $total')
            file_mappings = files_by_queue[queue_directory]
            for ext in sorted(file_mappings):
                print '\t{0}: {1}'.format(ext, file_mappings[ext])
        return
    # No summary.
    for filename in options.arguments:
        if not options.options.quiet:
            print '====================>', filename
        with open(filename) as fp:
            if filename.endswith('.pck'):
                msg = load(fp)
                data = load(fp)
                if data.get('_parsemsg'):
                    sys.stdout.write(msg)
                else:
                    sys.stdout.write(msg.as_string())
            else:
                sys.stdout.write(fp.read())



if __name__ == '__main__':
    main()



# Copyright (C) 2008-2012 by the Free Software Foundation, Inc.
#
# This file is part of GNU Mailman.
#
# GNU Mailman is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# GNU Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# GNU Mailman.  If not, see <http://www.gnu.org/licenses/>.

"""Built-in pipelines."""

from __future__ import absolute_import, print_function, unicode_literals

__metaclass__ = type
__all__ = [
    'BasePipeline',
    'OwnerPipeline',
    'PostingPipeline',
    'VirginPipeline',
    'initialize',
    'process',
    ]


import logging

from zope.interface import implements
from zope.interface.verify import verifyObject

from mailman.app.bounces import bounce_message
from mailman.app.finder import find_components
from mailman.config import config
from mailman.core import errors
from mailman.core.i18n import _
from mailman.interfaces.handler import IHandler
from mailman.interfaces.pipeline import IPipeline

dlog = logging.getLogger('mailman.debug')
vlog = logging.getLogger('mailman.vette')



def process(mlist, msg, msgdata, pipeline_name='built-in'):
    """Process the message through the given pipeline.

    :param mlist: the IMailingList for this message.
    :param msg: The Message object.
    :param msgdata: The message metadata dictionary.
    :param pipeline_name: The name of the pipeline to process through.
    """
    message_id = msg.get('message-id', 'n/a')
    pipeline = config.pipelines[pipeline_name]
    for handler in pipeline:
        dlog.debug('{0} pipeline {1} processing: {2}'.format(
            message_id, pipeline_name, handler.name))
        try:
            handler.process(mlist, msg, msgdata)
        except errors.DiscardMessage as error:
            vlog.info(
                '{0} discarded by "{1}" pipeline handler "{2}": {3}'.format(
                    message_id, pipeline_name, handler.name, error.message))
        except errors.RejectMessage as error:
            vlog.info(
                '{0} rejected by "{1}" pipeline handler "{2}": {3}'.format(
                    message_id, pipeline_name, handler.name, error.message))
            bounce_message(mlist, msg, error)



class BasePipeline:
    """Base pipeline implementation."""

    implements(IPipeline)

    _default_handlers = ()

    def __init__(self):
        self._handlers = []
        for handler_name in self._default_handlers:
            self._handlers.append(config.handlers[handler_name])

    def __iter__(self):
        """See `IPipeline`."""
        for handler in self._handlers:
            yield handler



class OwnerPipeline(BasePipeline):
    """The built-in owner pipeline."""

    name = 'default-owner-pipeline'
    description = _('The built-in owner pipeline.')

    _default_handlers = (
        'owner-recipients',
        'to-outgoing',
        )


class PostingPipeline(BasePipeline):
    """The built-in posting pipeline."""

    name = 'default-posting-pipeline'
    description = _('The built-in posting pipeline.')

    _default_handlers = (
        'mime-delete',
        'tagger',
        'member-recipients',
        'avoid-duplicates',
        'cleanse',
        'cleanse-dkim',
        'cook-headers',
        'rfc-2369',
        'to-archive',
        'to-digest',
        'to-usenet',
        'after-delivery',
        'acknowledge',
        'to-outgoing',
        )


class VirginPipeline(BasePipeline):
    """The processing pipeline for virgin messages.

    Virgin messages are those that are crafted internally by Mailman.
    """
    name = 'virgin'
    description = _('The virgin queue pipeline.')

    _default_handlers = (
        'cook-headers',
        'to-outgoing',
        )



def initialize():
    """Initialize the pipelines."""
    # Find all handlers in the registered plugins.
    for handler_class in find_components('mailman.handlers', IHandler):
        handler = handler_class()
        verifyObject(IHandler, handler)
        assert handler.name not in config.handlers, (
            'Duplicate handler "{0}" found in {1}'.format(
                handler.name, handler_class))
        config.handlers[handler.name] = handler
    # Set up some pipelines.
    for pipeline_class in (OwnerPipeline, PostingPipeline, VirginPipeline):
        pipeline = pipeline_class()
        config.pipelines[pipeline.name] = pipeline



# Copyright (C) 1998-2012 by the Free Software Foundation, Inc.
#
# This file is part of GNU Mailman.
#
# GNU Mailman is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# GNU Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# GNU Mailman.  If not, see <http://www.gnu.org/licenses/>.

"""Standard Mailman message object.

This is a subclass of email.message.Message but provides a slightly extended
interface which is more convenient for use inside Mailman.  It also supports
safe pickle deserialization, even if the email package adds additional Message
attributes.
"""

from __future__ import absolute_import, unicode_literals

__metaclass__ = type
__all__ = [
    'Message',
    'OwnerNotification',
    'UserNotification',
    ]


import email
import email.message
import email.utils

from email.header import Header

from mailman.config import config


COMMASPACE = ', '
VERSION = tuple(int(v) for v in email.__version__.split('.'))



class Message(email.message.Message):
    def __init__(self):
        # We need a version number so that we can optimize __setstate__().
        self.__version__ = VERSION
        email.message.Message.__init__(self)

    def __getitem__(self, key):
        # Ensure that header values are unicodes.
        value = email.message.Message.__getitem__(self, key)
        if isinstance(value, str):
            return unicode(value, 'ascii')
        return value

    def get(self, name, failobj=None):
        # Ensure that header values are unicodes.
        value = email.message.Message.get(self, name, failobj)
        if isinstance(value, str):
            return unicode(value, 'ascii')
        return value

    def get_all(self, name, failobj=None):
        # Ensure all header values are unicodes.
        missing = object()
        all_values = email.message.Message.get_all(self, name, missing)
        if all_values is missing:
            return failobj
        return [(unicode(value, 'ascii') if isinstance(value, str) else value)
                for value in all_values]

    # BAW: For debugging w/ bin/dumpdb.  Apparently pprint uses repr.
    def __repr__(self):
        return self.__str__()

    def __setstate__(self, values):
        # The base class has grown and changed attributes over time.  This can
        # break messages sitting in Mailman's queues at the time of upgrading
        # the email package.  We can't (yet) change the email package to be
        # safer for pickling, so we handle such changes here.  Note that we're
        # using Python 2.6's email package version 4.0.1 as a base line here.
        self.__dict__ = values
        # The pickled instance should have an __version__ string, but it may
        # not if it's an email package message.
        version = values.get('__version__', (0, 0, 0))
        values['__version__'] = version
        # There's really nothing to check; there's nothing newer than email
        # 4.0.1 at the moment.

    @property
    def sender(self):
        """The address considered to be the author of the email.

        This is the first non-None value in the list of senders.

        :return: The email address of the first found sender, or the empty
            string if no sender address was found.
        :rtype: email address
        """
        for address in self.senders:
            # This could be None or the empty string.
            if address:
                return address
        return ''

    @property
    def senders(self):
        """Return a list of addresses representing the author of the email.

        The list will contain email addresses in the order determined by the
        configuration variable `sender_headers` in the `[mailman]` section.
        By default it uses this list of headers in order:

        1. From:
        2. envelope sender (i.e. From_, unixfrom, or RFC 2821 MAIL FROM)
        3. Reply-To:
        4. Sender:

        The return addresses are guaranteed to be lower case or None.  There
        may be more than four values in the returned list, since some of the
        originator headers above can appear multiple times in the message, or
        contain multiple values.

        :return: The list of email addresses that can be considered the sender
            of the message.
        :rtype: A list of email addresses or Nones
        """
        envelope_sender = self.get_unixfrom()
        senders = []
        for header in config.mailman.sender_headers.split():
            header = header.lower()
            if header == 'from_':
                senders.append(envelope_sender.lower()
                               if envelope_sender is not None
                               else '')
            else:
                field_values = self.get_all(header, [])
                senders.extend(address.lower() for (display_name, address)
                               in email.utils.getaddresses(field_values))
        # Filter out None and the empty string.
        return [sender for sender in senders if sender]

    def get_filename(self, failobj=None):
        """Some MUA have bugs in RFC2231 filename encoding and cause
        Mailman to stop delivery in Scrubber.py (called from ToDigest.py).
        """
        try:
            filename = email.message.Message.get_filename(self, failobj)
            return filename
        except (UnicodeError, LookupError, ValueError):
            return failobj



class UserNotification(Message):
    """Class for internally crafted messages."""

    def __init__(self, recipients, sender, subject=None, text=None, lang=None):
        Message.__init__(self)
        charset = (lang.charset if lang is not None else 'us-ascii')
        subject = ('(no subject)' if subject is None else subject)
        if text is not None:
            self.set_payload(text.encode(charset), charset)
        self['Subject'] = Header(subject.encode(charset), charset,
                                 header_name='Subject', errors='replace')
        self['From'] = sender
        if isinstance(recipients, (list, set, tuple)):
            self['To'] = COMMASPACE.join(recipients)
            self.recipients = recipients
        else:
            self['To'] = recipients
            self.recipients = set([recipients])

    def send(self, mlist, add_precedence=True, **_kws):
        """Sends the message by enqueuing it to the 'virgin' queue.

        This is used for all internally crafted messages.

        :param mlist: The mailing list to send the message to.
        :type mlist: `IMailingList`
        :param add_precedence: Flag indicating whether a `Precedence: bulk` 
            header should be added to the message or not.
        :type add_precedence: bool

        This function also accepts arbitrary keyword arguments.  The key/value
        pairs for **kws is added to the metadata dictionary associated with
        the enqueued message.
        """
        # Since we're crafting the message from whole cloth, let's make sure
        # this message has a Message-ID.
        if 'message-id' not in self:
            self['Message-ID'] = email.utils.make_msgid()
        # Ditto for Date: as required by RFC 2822.
        if 'date' not in self:
            self['Date'] = email.utils.formatdate(localtime=True)
        # UserNotifications are typically for admin messages, and for messages
        # other than list explosions.  Send these out as Precedence: bulk, but
        # don't override an existing Precedence: header.
        if 'precedence' not in self and add_precedence:
            self['Precedence'] = 'bulk'
        self._enqueue(mlist, **_kws)

    def _enqueue(self, mlist, **_kws):
        # Not imported at module scope to avoid import loop
        virginq = config.switchboards['virgin']
        # The message metadata better have a 'recip' attribute.
        enqueue_kws = dict(
            recipients=self.recipients,
            nodecorate=True,
            reduced_list_headers=True,
            )
        if mlist is not None:
            enqueue_kws['listname'] = mlist.fqdn_listname
        enqueue_kws.update(_kws)
        # Keywords must be strings in Python 2.6.
        str_keywords = dict()
        for key, val in enqueue_kws.items():
            str_keywords[str(key)] = val
        virginq.enqueue(self, **str_keywords)



class OwnerNotification(UserNotification):
    """Like user notifications, but this message goes to some owners."""

    def __init__(self, mlist, subject=None, text=None, roster=None):
        if roster is None:
            recipients = set([config.mailman.site_owner])
            to = config.mailman.site_owner
        else:
            recipients = set(address.email for address in roster.addresses)
            to = mlist.owner_address
        sender = config.mailman.site_owner
        UserNotification.__init__(self, recipients, sender, subject,
                                  text, mlist.preferred_language)
        # Hack the To header to look like it's going to the -owner address
        del self['to']
        self['To'] = to
        self._sender = sender

    def _enqueue(self, mlist, **_kws):
        # Not imported at module scope to avoid import loop
        virginq = config.switchboards['virgin']
        # The message metadata better have a `recip' attribute
        virginq.enqueue(self,
                        listname=mlist.fqdn_listname,
                        recipients=self.recipients,
                        nodecorate=True,
                        reduced_list_headers=True,
                        envsender=self._sender,
                        **_kws)



# Copyright (C) 2007-2012 by the Free Software Foundation, Inc.
#
# This file is part of GNU Mailman.
#
# GNU Mailman is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# GNU Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# GNU Mailman.  If not, see <http://www.gnu.org/licenses/>.

"""Storm type conversions."""


from __future__ import absolute_import, unicode_literals

__metaclass__ = type
__all__ = [
    'Enum',
    ]


from storm.properties import SimpleProperty
from storm.variables import Variable



class _EnumVariable(Variable):
    """Storm variable for supporting flufl.enum.Enum types.

    To use this, make the database column a INTEGER.
    """

    def __init__(self, *args, **kws):
        self._enum = kws.pop('enum')
        super(_EnumVariable, self).__init__(*args, **kws)

    def parse_set(self, value, from_db):
        if value is None:
            return None
        if not from_db:
            return value
        return self._enum[value]

    def parse_get(self, value, to_db):
        if value is None:
            return None
        if not to_db:
            return value
        return int(value)


class Enum(SimpleProperty):
    """Custom Enum type for Storm supporting flufl.enum.Enums."""

    variable_class = _EnumVariable

    def __init__(self, enum=None):
        super(Enum, self).__init__(enum=enum)



# Copyright (C) 2009-2012 by the Free Software Foundation, Inc.
#
# This file is part of GNU Mailman.
#
# GNU Mailman is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# GNU Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# GNU Mailman.  If not, see <http://www.gnu.org/licenses/>.

"""bin/mailman withlist"""

from __future__ import absolute_import, unicode_literals

__metaclass__ = type
__all__ = [
    'Shell',
    'Withlist',
    ]


import re
import sys

from lazr.config import as_boolean
from zope.component import getUtility
from zope.interface import implements

from mailman.config import config
from mailman.core.i18n import _
from mailman.interfaces.command import ICLISubCommand
from mailman.interfaces.listmanager import IListManager
from mailman.utilities.interact import DEFAULT_BANNER, interact
from mailman.utilities.modules import call_name

# Global holding onto the open mailing list.
m = None
# Global holding the results of --run.
r = None



class Withlist:
    """Operate on a mailing list.

    For detailed help, see --details
    """

    implements(ICLISubCommand)

    name = 'withlist'

    def add(self, parser, command_parser):
        """See `ICLISubCommand`."""
        self.parser = parser
        command_parser.add_argument(
            '-i', '--interactive',
            default=None, action='store_true', help=_("""\
            Leaves you at an interactive prompt after all other processing is
            complete.  This is the default unless the --run option is
            given."""))
        command_parser.add_argument(
            '-r', '--run',
            help=_("""\
            Run a script on a mailing list.  The argument is the module path
            to a callable.  This callable will be imported and then called
            with the mailing list as the first argument.  If additional
            arguments are given at the end of the command line, they are
            passed as subsequent positional arguments to the callable.  For
            additional help, see --details.
            """))
        command_parser.add_argument(
            '--details',
            default=False, action='store_true',
            help=_('Print detailed instructions on using this command.'))
        # Optional positional argument.
        command_parser.add_argument(
            'listname', metavar='LISTNAME', nargs='?',
            help=_("""\
            The 'fully qualified list name', i.e. the posting address of the
            mailing list to inject the message into.  This can be a Python
            regular expression, in which case all mailing lists whose posting
            address matches will be processed.  To use a regular expression,
            LISTNAME must start with a ^ (and the matching is done with
            re.match().  LISTNAME cannot be a regular expression unless --run
            is given."""))

    def process(self, args):
        """See `ICLISubCommand`."""
        global m, r
        banner = DEFAULT_BANNER
        # Detailed help wanted?
        if args.details:
            self._details()
            return
        # Interactive is the default unless --run was given.
        if args.interactive is None:
            interactive = (args.run is None)
        else:
            interactive = args.interactive
        # List name cannot be a regular expression if --run is not given.
        if args.listname and args.listname.startswith('^') and not args.run:
            self.parser.error(_('Regular expression requires --run'))
            return
        # Handle --run.
        list_manager = getUtility(IListManager)
        if args.run:
            # When the module and the callable have the same name, a shorthand
            # without the dot is allowed.
            dotted_name = (args.run if '.' in args.run
                           else '{0}.{0}'.format(args.run))
            if args.listname is None:
                self.parser.error(_('--run requires a mailing list name'))
                return
            elif args.listname.startswith('^'):
                r = {}
                cre = re.compile(args.listname, re.IGNORECASE)
                for mailing_list in list_manager.mailing_lists:
                    if cre.match(mailing_list.fqdn_listname):
                        results = call_name(dotted_name, mailing_list)
                        r[mailing_list.fqdn_listname] = results
            else:
                fqdn_listname = args.listname
                m = list_manager.get(fqdn_listname)
                if m is None:
                    self.parser.error(_('No such list: $fqdn_listname'))
                    return
                r = call_name(dotted_name, m)
        else:
            # Not --run.
            if args.listname is not None:
                fqdn_listname = args.listname
                m = list_manager.get(fqdn_listname)
                if m is None:
                    self.parser.error(_('No such list: $fqdn_listname'))
                    return
                banner = _(
                    "The variable 'm' is the $fqdn_listname mailing list")
        # All other processing is finished; maybe go into interactive mode.
        if interactive:
            overrides = dict(
                m=m,
                commit=config.db.commit,
                abort=config.db.abort,
                config=config,
                )
            banner = config.shell.banner + '\n' + banner
            if as_boolean(config.shell.use_ipython):
                self._start_ipython(overrides, banner)
            else:
                self._start_python(overrides, banner)

    def _start_ipython(self, overrides, banner):
        try:
            from IPython.frontend.terminal.embed import InteractiveShellEmbed
            ipshell = InteractiveShellEmbed(banner1=banner, user_ns=overrides)
            ipshell()
        except ImportError:
            print _('ipython is not available, set use_ipython to no')

    def _start_python(self, overrides, banner):
        # Set the tab completion.
        try:
            import readline, rlcompleter
            readline.parse_and_bind('tab: complete')
        except ImportError:
            pass
        else:
            sys.ps1 = config.shell.prompt + ' '
        interact(upframe=False, banner=banner, overrides=overrides)

    def _details(self):
        """Print detailed usage."""
        # Split this up into paragraphs for easier translation.
        print _("""\
This script provides you with a general framework for interacting with a
mailing list.""")
        print
        print _("""\
There are two ways to use this script: interactively or programmatically.
Using it interactively allows you to play with, examine and modify a mailing
list from Python's interactive interpreter.  When running interactively, the
variable 'm' will be available in the global namespace.  It will reference the
mailing list object.""")
        print
        print _("""\
Programmatically, you can write a function to operate on a mailing list, and
this script will take care of the housekeeping (see below for examples).  In
that case, the general usage syntax is:

    % bin/mailman withlist [options] listname [args ...]""")
        print
        print _("""\
Here's an example of how to use the --run option.  Say you have a file in the
Mailman installation directory called 'listaddr.py', with the following two
functions:

    def listaddr(mlist):
        print mlist.posting_address

    def requestaddr(mlist):
        print mlist.request_address""")
        print
        print _("""\
You can print the list's posting address by running the following from the
command line:

    % bin/mailman withlist -r listaddr mylist@example.com
    Importing listaddr ...
    Running listaddr.listaddr() ...
    mylist@example.com""")
        print
        print _("""\
And you can print the list's request address by running:

    % bin/mailman withlist -r listaddr.requestaddr mylist
    Importing listaddr ...
    Running listaddr.requestaddr() ...
    mylist-request@example.com""")
        print
        print _("""\
As another example, say you wanted to change the display name for a particular
mailing list.  You could put the following function in a file called
'change.pw':

    def change(mlist, display_name):
        mlist.display_name = display_name
        # Required to save changes to the database.
        commit()

and run this from the command line:

    % bin/mailman withlist -r change mylist@example.com 'My List'""")



class Shell(Withlist):
    """An alias for `withlist`."""

    name = 'shell'



# Copyright (C) 2011-2012 by the Free Software Foundation, Inc.
#
# This file is part of GNU Mailman.
#
# GNU Mailman is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# GNU Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# GNU Mailman.  If not, see <http://www.gnu.org/licenses/>.

"""Test some additional corner cases for starting/stopping."""

from __future__ import absolute_import, unicode_literals

__metaclass__ = type
__all__ = [
    'find_master',
    'make_config',
    ]


import os
import sys
import time
import errno
import signal
import shutil
import socket
import unittest

from datetime import timedelta, datetime

from mailman.commands.cli_control import Start, kill_watcher
from mailman.config import config
from mailman.testing.layers import ConfigLayer

SEP = '|'



def make_config():
    # All we care about is the master process; normally it starts a bunch of
    # runners, but we don't care about any of them, so write a test
    # configuration file for the master that disables all the runners.
    new_config = 'no-runners.cfg'
    config_file = os.path.join(os.path.dirname(config.filename), new_config)
    shutil.copyfile(config.filename, config_file)
    with open(config_file, 'a') as fp:
        for runner_config in config.runner_configs:
            print >> fp, '[{0}]\nstart:no\n'.format(runner_config.name)
    return config_file


def find_master():
    # See if the master process is still running.
    until = timedelta(seconds=10) + datetime.now()
    while datetime.now() < until:
        time.sleep(0.1)
        try:
            with open(config.PID_FILE) as fp:
                pid = int(fp.read().strip())
                os.kill(pid, 0)
        except IOError as error:
            if error.errno != errno.ENOENT:
                raise
        except ValueError:
            pass
        except OSError as error:
            if error.errno != errno.ESRCH:
                raise
        else:
            return pid
    else:
        return None



class FakeArgs:
    force = None
    run_as_user = None
    quiet = True
    config = None


class FakeParser:
    def __init__(self):
        self.message = None

    def error(self, message):
        self.message = message
        sys.exit(1)



class TestStart(unittest.TestCase):
    """Test various starting scenarios."""

    layer = ConfigLayer

    def setUp(self):
        self.command = Start()
        self.command.parser = FakeParser()
        self.args = FakeArgs()
        self.args.config = make_config()

    def tearDown(self):
        try:
            with open(config.PID_FILE) as fp:
                master_pid = int(fp.read())
        except OSError as error:
            if error.errno != errno.ENOENT:
                raise
            # There is no master, so just ignore this.
            return
        kill_watcher(signal.SIGTERM)
        os.waitpid(master_pid, 0)

    def test_force_stale_lock(self):
        # Fake an acquisition of the master lock by another process, which
        # subsequently goes stale.  Start by finding a free process id.  Yes,
        # this could race, but given that we're starting with our own PID and
        # searching downward, it's less likely.
        fake_pid = os.getpid() - 1
        while fake_pid > 1:
            try:
                os.kill(fake_pid, 0)
            except OSError as error:
                if error.errno == errno.ESRCH:
                    break
            fake_pid -= 1
        else:
            raise RuntimeError('Cannot find free PID')
        # Lock acquisition logic taken from flufl.lock.
        claim_file = SEP.join((
            config.LOCK_FILE,
            socket.getfqdn(),
            str(fake_pid),
            '0'))
        with open(config.LOCK_FILE, 'w') as fp:
            fp.write(claim_file)
        os.link(config.LOCK_FILE, claim_file)
        expiration_date = datetime.now() - timedelta(minutes=2)
        t = time.mktime(expiration_date.timetuple())
        os.utime(claim_file, (t, t))
        # Start without --force; no master will be running.
        try:
            self.command.process(self.args)
        except SystemExit:
            pass
        self.assertEqual(find_master(), None)
        self.assertTrue('--force' in self.command.parser.message)
        # Start again, this time with --force.
        self.args.force = True
        self.command.process(self.args)
        pid = find_master()
        self.assertNotEqual(pid, None)



# Copyright (C) 2011-2012 by the Free Software Foundation, Inc.
#
# This file is part of GNU Mailman.
#
# GNU Mailman is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# GNU Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# GNU Mailman.  If not, see <http://www.gnu.org/licenses/>.

"""Tests of application level membership functions."""

from __future__ import absolute_import, unicode_literals

__metaclass__ = type
__all__ = [
    ]


import unittest

from zope.component import getUtility

from mailman.app.lifecycle import create_list
from mailman.app.membership import add_member
from mailman.config import config
from mailman.core.constants import system_preferences
from mailman.interfaces.bans import IBanManager
from mailman.interfaces.member import (
    AlreadySubscribedError, DeliveryMode, MemberRole, MembershipIsBannedError)
from mailman.interfaces.usermanager import IUserManager
from mailman.testing.helpers import reset_the_world
from mailman.testing.layers import ConfigLayer



class AddMemberTest(unittest.TestCase):
    layer = ConfigLayer

    def setUp(self):
        self._mlist = create_list('test@example.com')

    def tearDown(self):
        reset_the_world()

    def test_add_member_new_user(self):
        # Test subscribing a user to a mailing list when the email address has
        # not yet been associated with a user.
        member = add_member(self._mlist, 'aperson@example.com',
                            'Anne Person', '123', DeliveryMode.regular,
                            system_preferences.preferred_language)
        self.assertEqual(member.address.email, 'aperson@example.com')
        self.assertEqual(member.mailing_list, 'test@example.com')
        self.assertEqual(member.role, MemberRole.member)

    def test_add_member_existing_user(self):
        # Test subscribing a user to a mailing list when the email address has
        # already been associated with a user.
        user_manager = getUtility(IUserManager)
        user_manager.create_user('aperson@example.com', 'Anne Person')
        member = add_member(self._mlist, 'aperson@example.com',
                            'Anne Person', '123', DeliveryMode.regular,
                            system_preferences.preferred_language)
        self.assertEqual(member.address.email, 'aperson@example.com')
        self.assertEqual(member.mailing_list, 'test@example.com')

    def test_add_member_banned(self):
        # Test that members who are banned by specific address cannot
        # subscribe to the mailing list.
        getUtility(IBanManager).ban('anne@example.com', 'test@example.com')
        self.assertRaises(
            MembershipIsBannedError,
            add_member, self._mlist, 'anne@example.com', 'Anne Person',
            '123', DeliveryMode.regular, system_preferences.preferred_language)

    def test_add_member_globally_banned(self):
        # Test that members who are banned by specific address cannot
        # subscribe to the mailing list.
        getUtility(IBanManager).ban('anne@example.com')
        self.assertRaises(
            MembershipIsBannedError,
            add_member, self._mlist, 'anne@example.com', 'Anne Person',
            '123', DeliveryMode.regular, system_preferences.preferred_language)

    def test_add_member_banned_from_different_list(self):
        # Test that members who are banned by specific address cannot
        # subscribe to the mailing list.
        getUtility(IBanManager).ban('anne@example.com', 'sample@example.com')
        member = add_member(self._mlist, 'anne@example.com',
                            'Anne Person', '123', DeliveryMode.regular,
                            system_preferences.preferred_language)
        self.assertEqual(member.address.email, 'anne@example.com')

    def test_add_member_banned_by_pattern(self):
        # Test that members who are banned by specific address cannot
        # subscribe to the mailing list.
        getUtility(IBanManager).ban('^.*@example.com', 'test@example.com')
        self.assertRaises(
            MembershipIsBannedError,
            add_member, self._mlist, 'anne@example.com', 'Anne Person',
            '123', DeliveryMode.regular, system_preferences.preferred_language)

    def test_add_member_globally_banned_by_pattern(self):
        # Test that members who are banned by specific address cannot
        # subscribe to the mailing list.
        getUtility(IBanManager).ban('^.*@example.com')
        self.assertRaises(
            MembershipIsBannedError,
            add_member, self._mlist, 'anne@example.com', 'Anne Person',
            '123', DeliveryMode.regular, system_preferences.preferred_language)

    def test_add_member_banned_from_different_list_by_pattern(self):
        # Test that members who are banned by specific address cannot
        # subscribe to the mailing list.
        getUtility(IBanManager).ban('^.*@example.com', 'sample@example.com')
        member = add_member(self._mlist, 'anne@example.com',
                            'Anne Person', '123', DeliveryMode.regular,
                            system_preferences.preferred_language)
        self.assertEqual(member.address.email, 'anne@example.com')

    def test_add_member_moderator(self):
        # Test adding a moderator to a mailing list.
        member = add_member(self._mlist, 'aperson@example.com',
                            'Anne Person', '123', DeliveryMode.regular,
                            system_preferences.preferred_language,
                            MemberRole.moderator)
        self.assertEqual(member.address.email, 'aperson@example.com')
        self.assertEqual(member.mailing_list, 'test@example.com')
        self.assertEqual(member.role, MemberRole.moderator)
    
    def test_add_member_twice(self):
        # Adding a member with the same role twice causes an
        # AlreadySubscribedError to be raised.
        add_member(self._mlist, 'aperson@example.com',
                   'Anne Person', '123', DeliveryMode.regular,
                   system_preferences.preferred_language,
                   MemberRole.member)
        try:
            add_member(self._mlist, 'aperson@example.com',
                       'Anne Person', '123', DeliveryMode.regular,
                       system_preferences.preferred_language,
                       MemberRole.member)
        except AlreadySubscribedError as exc:
            self.assertEqual(exc.fqdn_listname, 'test@example.com')
            self.assertEqual(exc.email, 'aperson@example.com')
            self.assertEqual(exc.role, MemberRole.member)
        else:
            raise AssertionError('AlreadySubscribedError expected')

    def test_add_member_with_different_roles(self):
        # Adding a member twice with different roles is okay.
        member_1 = add_member(self._mlist, 'aperson@example.com',
                              'Anne Person', '123', DeliveryMode.regular,
                              system_preferences.preferred_language,
                              MemberRole.member)
        member_2 = add_member(self._mlist, 'aperson@example.com',
                              'Anne Person', '123', DeliveryMode.regular,
                              system_preferences.preferred_language,
                              MemberRole.owner)
        self.assertEqual(member_1.mailing_list, member_2.mailing_list)
        self.assertEqual(member_1.address, member_2.address)
        self.assertEqual(member_1.user, member_2.user)
        self.assertNotEqual(member_1.member_id, member_2.member_id)
        self.assertEqual(member_1.role, MemberRole.member)
        self.assertEqual(member_2.role, MemberRole.owner)



class AddMemberPasswordTest(unittest.TestCase):
    layer = ConfigLayer

    def setUp(self):
        self._mlist = create_list('test@example.com')
        # The default ssha scheme introduces a random salt, which is
        # inappropriate for unit tests.
        config.push('password scheme', """
        [passwords]
        password_scheme: sha
        """)

    def tearDown(self):
        config.pop('password scheme')
        reset_the_world()

    def test_add_member_password(self):
        # Test that the password stored with the new user is encrypted.
        member = add_member(self._mlist, 'anne@example.com',
                            'Anne Person', 'abc', DeliveryMode.regular,
                            system_preferences.preferred_language)
        self.assertEqual(
            member.user.password, '{SHA}qZk-NkcGgWq6PiVxeFDCbJzQ2J0=')



# Copyright (C) 2007-2012 by the Free Software Foundation, Inc.
#
# This file is part of GNU Mailman.
#
# GNU Mailman is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# GNU Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# GNU Mailman.  If not, see <http://www.gnu.org/licenses/>.

"""Language manager."""

from __future__ import absolute_import, unicode_literals

__metaclass__ = type
__all__ = [
    'LanguageManager',
    ]

from zope.interface import implements

from mailman.interfaces.languages import ILanguageManager
from mailman.languages.language import Language



class LanguageManager:
    """Language manager."""

    implements(ILanguageManager)

    def __init__(self):
        # Mapping from 2-letter code to Language instance.
        self._languages = {}

    def add(self, code, charset, description):
        """See `ILanguageManager`."""
        if code in self._languages:
            raise ValueError('Language code already registered: ' + code)
        language = Language(code, charset, description)
        self._languages[code] = language
        return language

    @property
    def codes(self):
        """See `ILanguageManager`."""
        return iter(self._languages)

    @property
    def languages(self):
        """See `ILanguageManager`."""
        return iter(self._languages.values())

    def get(self, code, default=None):
        """See `ILanguageManager`."""
        return self._languages.get(code, default)

    def __getitem__(self, code):
        """See `ILanguageManager`."""
        return self._languages[code]

    def __contains__(self, code):
        """See `ILanguageManager`."""
        return code in self._languages

    def clear(self):
        """See `ILanguageManager`."""
        self._languages.clear()



# Copyright (C) 2007-2012 by the Free Software Foundation, Inc.
#
# This file is part of GNU Mailman.
#
# GNU Mailman is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# GNU Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# GNU Mailman.  If not, see <http://www.gnu.org/licenses/>.

"""Harness for testing Mailman's documentation.

Note that doctest extraction does not currently work for zip file
distributions.  doctest discovery currently requires file system traversal.
"""

from __future__ import absolute_import, unicode_literals

__metaclass__ = type
__all__ = [
    'test_suite',
    ]


import os
import sys
import doctest
import unittest

import mailman

from mailman.app.lifecycle import create_list
from mailman.config import config
from mailman.testing.helpers import call_api, specialized_message_from_string
from mailman.testing.layers import SMTPLayer


DOT = '.'



class chdir:
    """A context manager for temporary directory changing."""
    def __init__(self, directory):
        self._curdir = None
        self._directory = directory

    def __enter__(self):
        self._curdir = os.getcwd()
        os.chdir(self._directory)

    def __exit__(self, exc_type, exc_val, exc_tb):
        os.chdir(self._curdir)
        # Don't suppress exceptions.
        return False



def stop():
    """Call into pdb.set_trace()"""
    # Do the import here so that you get the wacky special hacked pdb instead
    # of Python's normal pdb.
    import pdb
    pdb.set_trace()


def dump_msgdata(msgdata, *additional_skips):
    """Dump in a more readable way a message metadata dictionary."""
    if len(msgdata) == 0:
        print '*Empty*'
        return
    skips = set(additional_skips)
    # Some stuff we always want to skip, because their values will always be
    # variable data.
    skips.add('received_time')
    longest = max(len(key) for key in msgdata if key not in skips)
    for key in sorted(msgdata):
        if key in skips:
            continue
        print '{0:{2}}: {1}'.format(key, msgdata[key], longest)


def dump_list(list_of_things, key=str):
    """Print items in a string to get rid of stupid u'' prefixes."""
    # List of things may be a generator.
    list_of_things = list(list_of_things)
    if len(list_of_things) == 0:
        print '*Empty*'
    if key is not None:
        list_of_things = sorted(list_of_things, key=key)
    for item in list_of_things:
        print item


def call_http(url, data=None, method=None, username=None, password=None):
    """'Call a URL with a given HTTP method and return the resulting object.

    The object will have been JSON decoded.

    :param url: The url to open, read, and print.
    :type url: string
    :param data: Data to use to POST to a URL.
    :type data: dict
    :param method: Alternative HTTP method to use.
    :type method: str
    :param username: The HTTP Basic Auth user name.  None means use the value
        from the configuration.
    :type username: str
    :param password: The HTTP Basic Auth password.  None means use the value
        from the configuration.
    :type username: str
    :return: The decoded JSON data structure.
    :raises HTTPError: when a non-2xx return code is received.
    """
    content, response = call_api(url, data, method, username, password)
    if content is None:
        for header in sorted(response):
            print '{0}: {1}'.format(header, response[header])
        return None
    return content


def dump_json(url, data=None, method=None, username=None, password=None):
    """Print the JSON dictionary read from a URL.

    :param url: The url to open, read, and print.
    :type url: string
    :param data: Data to use to POST to a URL.
    :type data: dict
    :param method: Alternative HTTP method to use.
    :type method: str
    :param username: The HTTP Basic Auth user name.  None means use the value
        from the configuration.
    :type username: str
    :param password: The HTTP Basic Auth password.  None means use the value
        from the configuration.
    :type username: str
    """
    results = call_http(url, data, method, username, password)
    if results is None:
        return
    for key in sorted(results):
        if key == 'entries':
            for i, entry in enumerate(results[key]):
                # entry is a dictionary.
                print 'entry %d:' % i
                for entry_key in sorted(entry):
                    print '    {0}: {1}'.format(entry_key, entry[entry_key])
        else:
            print '{0}: {1}'.format(key, results[key])



def setup(testobj):
    """Test setup."""
    # Make sure future statements in our doctests are the same as everywhere
    # else.
    testobj.globs['absolute_import'] = absolute_import
    testobj.globs['unicode_literals'] = unicode_literals
    # In general, I don't like adding convenience functions, since I think
    # doctests should do the imports themselves.  It makes for better
    # documentation that way.  However, a few are really useful, or help to
    # hide some icky test implementation details.
    testobj.globs['call_http'] = call_http
    testobj.globs['config'] = config
    testobj.globs['create_list'] = create_list
    testobj.globs['dump_json'] = dump_json
    testobj.globs['dump_msgdata'] = dump_msgdata
    testobj.globs['dump_list'] = dump_list
    testobj.globs['message_from_string'] = specialized_message_from_string
    testobj.globs['smtpd'] = SMTPLayer.smtpd
    testobj.globs['stop'] = stop
    testobj.globs['transaction'] = config.db



def test_suite():
    """Create test suites for all .rst documentation tests.

    .txt files are also tested, but .rst is highly preferred.
    """
    suite = unittest.TestSuite()
    topdir = os.path.dirname(mailman.__file__)
    packages = []
    for dirpath, dirnames, filenames in os.walk(topdir):
        if 'docs' in dirnames:
            docsdir = os.path.join(dirpath, 'docs')[len(topdir)+1:]
            packages.append(docsdir)
    # Under higher verbosity settings, report all doctest errors, not just the
    # first one.
    flags = (doctest.ELLIPSIS |
             doctest.NORMALIZE_WHITESPACE |
             doctest.REPORT_NDIFF)
    # Add all the doctests in all subpackages.
    doctest_files = {}
    with chdir(topdir):
        for docsdir in packages:
            # Look to see if the package defines a test layer, otherwise use
            # SMTPLayer.
            package_path = 'mailman.' + DOT.join(docsdir.split(os.sep))
            try:
                __import__(package_path)
            except ImportError:
                layer = SMTPLayer
            else:
                layer = getattr(sys.modules[package_path], 'layer', SMTPLayer)
            for filename in os.listdir(docsdir):
                base, extension = os.path.splitext(filename)
                if os.path.splitext(filename)[1] in ('.txt', '.rst'):
                    module_path = package_path + '.' + base
                    doctest_files[module_path] = (
                        os.path.join(docsdir, filename), layer)
    for module_path in sorted(doctest_files):
        path, layer = doctest_files[module_path]
        test = doctest.DocFileSuite(
            path,
            package='mailman',
            optionflags=flags,
            setUp=setup)
        test.layer = layer
        suite.addTest(test)
    return suite



# Copyright (C) 2009-2012 by the Free Software Foundation, Inc.
#
# This file is part of GNU Mailman.
#
# GNU Mailman is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# GNU Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# GNU Mailman.  If not, see <http://www.gnu.org/licenses/>.

"""The 'unshunt' command."""

from __future__ import absolute_import, unicode_literals

__metaclass__ = type
__all__ = [
    'Unshunt',
    ]


import sys

from zope.interface import implements

from mailman.config import config
from mailman.core.i18n import _
from mailman.interfaces.command import ICLISubCommand



class Unshunt:
    """Unshunt messages."""

    implements(ICLISubCommand)

    name = 'unshunt'

    def add(self, parser, command_parser):
        """See `ICLISubCommand`."""
        self.parser = parser
        command_parser.add_argument(
            '-d', '--discard',
            default=False, action='store_true',
            help=_("""\
            Discard all shunted messages instead of moving them back to their
            original queue."""))

    def process(self, args):
        """See `ICLISubCommand`."""
        shunt_queue = config.switchboards['shunt']
        shunt_queue.recover_backup_files()

        for filebase in shunt_queue.files:
            try:
                msg, msgdata = shunt_queue.dequeue(filebase)
                which_queue = msgdata.get('whichq', 'in')
                if not args.discard:
                    config.switchboards[which_queue].enqueue(msg, msgdata)
            except Exception as error:
                print >> sys.stderr, _(
                    'Cannot unshunt message $filebase, skipping:\n$error')
            else:
                # Unlink the .bak file left by dequeue()
                shunt_queue.finish(filebase)



# Copyright (C) 2003-2012 by the Free Software Foundation, Inc.
#
# This file is part of GNU Mailman.
#
# GNU Mailman is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# GNU Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# GNU Mailman.  If not, see <http://www.gnu.org/licenses/>.

"""Retry delivery."""

from __future__ import absolute_import, unicode_literals

__metaclass__ = type
__all__ = [
    'RetryRunner',
    ]


import time

from mailman.config import config
from mailman.core.runner import Runner



class RetryRunner(Runner):
    """Retry delivery."""

    def _dispose(self, mlist, msg, msgdata):
        # Move the message to the out queue for another try.
        config.switchboards['out'].enqueue(msg, msgdata)
        return False

    def _snooze(self, filecnt):
        # We always want to snooze.
        time.sleep(self.sleep_float)
